@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2007  The DOSBox Team
+ *  Copyright (C) 2002-2009  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: callback.h,v 1.21 2007/06/12 20:22:07 c2woody Exp $ */
+/* $Id: callback.h,v 1.24 2009/05/27 09:15:40 qbix79 Exp $ */
 
 #ifndef DOSBOX_CALLBACK_H
 #define DOSBOX_CALLBACK_H
@@ -30,7 +30,8 @@ extern CallBack_Handler CallBack_Handlers[];
 
 enum { CB_RETN,CB_RETF,CB_RETF8,CB_IRET,CB_IRETD,CB_IRET_STI,CB_IRET_EOI_PIC1,
 		CB_IRQ0,CB_IRQ1,CB_IRQ9,CB_IRQ12,CB_IRQ12_RET,CB_IRQ6_PCJR,CB_MOUSE,
-		CB_INT29,CB_INT16,CB_HOOKABLE,CB_TDE_IRET,CB_IPXESR,CB_IPXESR_RET };
+		CB_INT29,CB_INT16,CB_HOOKABLE,CB_TDE_IRET,CB_IPXESR,CB_IPXESR_RET,
+		CB_INT21 };
 
 #define CB_MAX 128
 #define CB_SIZE 32
@@ -42,14 +43,14 @@ enum {
 
 extern Bit8u lastint;
 
-INLINE RealPt CALLBACK_RealPointer(Bitu callback) {
+static INLINE RealPt CALLBACK_RealPointer(Bitu callback) {
 	return RealMake(CB_SEG,(Bit16u)(callback*CB_SIZE));
 }
-INLINE PhysPt CALLBACK_PhysPointer(Bitu callback) {
+static INLINE PhysPt CALLBACK_PhysPointer(Bitu callback) {
 	return PhysMake(CB_SEG,(Bit16u)(callback*CB_SIZE));
 }
 
-INLINE PhysPt CALLBACK_GetBase(void) {
+static INLINE PhysPt CALLBACK_GetBase(void) {
 	return CB_SEG << 4;
 }
 
