@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2004  The DOSBox Team
+ *  Copyright (C) 2002-2009  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,8 +16,9 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ifndef _TIMER_H_
-#define _TIMER_H_
+#ifndef DOSBOX_TIMER_H
+#define DOSBOX_TIMER_H
+
 /* underlying clock rate in HZ */
 #include <SDL.h>
 
@@ -35,4 +36,3 @@ void TIMER_DelTickHandler(TIMER_TickHandler handler);
 void TIMER_AddTick(void);
 
 #endif
-
