@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2006  The DOSBox Team
+ *  Copyright (C) 2002-2009  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@ LazyFlags lflags;
 /* CF     Carry Flag -- Set on high-order bit carry or borrow; cleared
           otherwise.
 */
-Bitu get_CF(void) {
+Bit32u get_CF(void) {
 
 	switch (lflags.type) {
 	case t_UNKNOWN:
@@ -43,7 +43,6 @@ Bitu get_CF(void) {
 	case t_DECd:
 	case t_MUL:
 		return GETFLAG(CF);
-		break;
 	case t_ADDb:	
 		return (lf_resb<lf_var1b);
 	case t_ADDw:	
@@ -129,7 +128,7 @@ Bitu get_CF(void) {
             four bits of   AL; cleared otherwise. Used for decimal
             arithmetic.
 */
-Bitu get_AF(void) {
+Bit32u get_AF(void) {
 	Bitu type=lflags.type;
 	switch (type) {
 	case t_UNKNOWN:
@@ -170,6 +169,18 @@ Bitu get_AF(void) {
 		return lf_var1w & 0x0f;
 	case t_NEGd:
 		return lf_var1d & 0x0f;
+	case t_SHLb:
+	case t_SHRb:
+	case t_SARb:
+		return lf_var2b & 0x1f;
+	case t_SHLw:
+	case t_SHRw:
+	case t_SARw:
+		return lf_var2w & 0x1f;
+	case t_SHLd:
+	case t_SHRd:
+	case t_SARd:
+		return lf_var2d & 0x1f;
 	case t_ORb:
 	case t_ORw:
 	case t_ORd:
@@ -182,15 +193,6 @@ Bitu get_AF(void) {
 	case t_TESTb:
 	case t_TESTw:
 	case t_TESTd:
-	case t_SHLb:
-	case t_SHLw:
-	case t_SHLd:
-	case t_SHRb:
-	case t_SHRw:
-	case t_SHRd:
-	case t_SARb:
-	case t_SARw:
-	case t_SARd:
 	case t_DSHLw:
 	case t_DSHLd:
 	case t_DSHRw:
@@ -207,7 +209,7 @@ Bitu get_AF(void) {
 /* ZF     Zero Flag -- Set if result is zero; cleared otherwise.
 */
 
-Bitu get_ZF(void) {
+Bit32u get_ZF(void) {
 	Bitu type=lflags.type;
 	switch (type) {
 	case t_UNKNOWN:
@@ -275,7 +277,7 @@ Bitu get_ZF(void) {
 /* SF     Sign Flag -- Set equal to high-order bit of result (0 is
             positive, 1 if negative).
 */
-Bitu get_SF(void) {
+Bit32u get_SF(void) {
 	Bitu type=lflags.type;
 	switch (type) {
 	case t_UNKNOWN:
@@ -341,7 +343,7 @@ Bitu get_SF(void) {
 	return false;
 
 }
-Bitu get_OF(void) {
+Bit32u get_OF(void) {
 	Bitu type=lflags.type;
 	switch (type) {
 	case t_UNKNOWN:
@@ -387,18 +389,24 @@ Bitu get_OF(void) {
 	case t_NEGd:
 		return (lf_var1d == 0x80000000);
 	case t_SHLb:
-	case t_SHRb:
 		return (lf_resb ^ lf_var1b) & 0x80;
 	case t_SHLw:
-	case t_SHRw:
 	case t_DSHRw:
 	case t_DSHLw:
 		return (lf_resw ^ lf_var1w) & 0x8000;
 	case t_SHLd:
-	case t_SHRd:
 	case t_DSHRd:
 	case t_DSHLd:
 		return (lf_resd ^ lf_var1d) & 0x80000000;
+	case t_SHRb:
+		if ((lf_var2b&0x1f)==1) return (lf_var1b > 0x80);
+		else return false;
+	case t_SHRw:
+		if ((lf_var2b&0x1f)==1) return (lf_var1w > 0x8000);
+		else return false;
+	case t_SHRd:
+		if ((lf_var2b&0x1f)==1) return (lf_var1d > 0x80000000);
+		else return false;
 	case t_ORb:
 	case t_ORw:
 	case t_ORd:
@@ -442,14 +450,14 @@ Bit16u parity_lookup[256] = {
   FLAG_PF, 0, 0, FLAG_PF, 0, FLAG_PF, FLAG_PF, 0, 0, FLAG_PF, FLAG_PF, 0, FLAG_PF, 0, 0, FLAG_PF
   };
 
-Bitu get_PF(void) {
+Bit32u get_PF(void) {
 	switch (lflags.type) {
 	case t_UNKNOWN:
 		return GETFLAG(PF);
 	default:
-		return	(parity_lookup[lf_resb]);;
+		return	(parity_lookup[lf_resb]);
 	};
-	return false;
+	return 0;
 }
 
 
@@ -486,6 +494,7 @@ Bitu FillFlags(void) {
 #define SETCF(NEWBIT) reg_flags=(reg_flags & ~FLAG_CF)|(NEWBIT);
 
 #define SET_FLAG SETFLAGBIT
+
 Bitu FillFlags(void) {
 	switch (lflags.type) {
 	case t_UNKNOWN:
@@ -597,6 +606,7 @@ Bitu FillFlags(void) {
 
 	case t_ORb:
 		SET_FLAG(CF,false);
+		SET_FLAG(AF,false);
 		DOFLAG_ZFb;
 		DOFLAG_SFb;
 		SET_FLAG(OF,false);
@@ -604,6 +614,7 @@ Bitu FillFlags(void) {
 		break;
 	case t_ORw:
 		SET_FLAG(CF,false);
+		SET_FLAG(AF,false);
 		DOFLAG_ZFw;
 		DOFLAG_SFw;
 		SET_FLAG(OF,false);
@@ -611,6 +622,7 @@ Bitu FillFlags(void) {
 		break;
 	case t_ORd:
 		SET_FLAG(CF,false);
+		SET_FLAG(AF,false);
 		DOFLAG_ZFd;
 		DOFLAG_SFd;
 		SET_FLAG(OF,false);
@@ -621,6 +633,7 @@ Bitu FillFlags(void) {
 	case t_TESTb:
 	case t_ANDb:
 		SET_FLAG(CF,false);
+		SET_FLAG(AF,false);
 		DOFLAG_ZFb;
 		DOFLAG_SFb;
 		SET_FLAG(OF,false);
@@ -629,6 +642,7 @@ Bitu FillFlags(void) {
 	case t_TESTw:
 	case t_ANDw:
 		SET_FLAG(CF,false);
+		SET_FLAG(AF,false);
 		DOFLAG_ZFw;
 		DOFLAG_SFw;
 		SET_FLAG(OF,false);
@@ -637,6 +651,7 @@ Bitu FillFlags(void) {
 	case t_TESTd:
 	case t_ANDd:
 		SET_FLAG(CF,false);
+		SET_FLAG(AF,false);
 		DOFLAG_ZFd;
 		DOFLAG_SFd;
 		SET_FLAG(OF,false);
@@ -646,6 +661,7 @@ Bitu FillFlags(void) {
 	
 	case t_XORb:
 		SET_FLAG(CF,false);
+		SET_FLAG(AF,false);
 		DOFLAG_ZFb;
 		DOFLAG_SFb;
 		SET_FLAG(OF,false);
@@ -653,6 +669,7 @@ Bitu FillFlags(void) {
 		break;
 	case t_XORw:
 		SET_FLAG(CF,false);
+		SET_FLAG(AF,false);
 		DOFLAG_ZFw;
 		DOFLAG_SFw;
 		SET_FLAG(OF,false);
@@ -660,6 +677,7 @@ Bitu FillFlags(void) {
 		break;
 	case t_XORd:
 		SET_FLAG(CF,false);
+		SET_FLAG(AF,false);
 		DOFLAG_ZFd;
 		DOFLAG_SFd;
 		SET_FLAG(OF,false);
@@ -674,6 +692,7 @@ Bitu FillFlags(void) {
 		DOFLAG_SFb;
 		SET_FLAG(OF,(lf_resb ^ lf_var1b) & 0x80);
 		DOFLAG_PF;
+		SET_FLAG(AF,(lf_var2b&0x1f));
 		break;
 	case t_SHLw:
 		if (lf_var2b>16) SET_FLAG(CF,false);
@@ -682,6 +701,7 @@ Bitu FillFlags(void) {
 		DOFLAG_SFw;
 		SET_FLAG(OF,(lf_resw ^ lf_var1w) & 0x8000);
 		DOFLAG_PF;
+		SET_FLAG(AF,(lf_var2w&0x1f));
 		break;
 	case t_SHLd:
 		SET_FLAG(CF,(lf_var1d >> (32 - lf_var2b)) & 1);
@@ -689,6 +709,7 @@ Bitu FillFlags(void) {
 		DOFLAG_SFd;
 		SET_FLAG(OF,(lf_resd ^ lf_var1d) & 0x80000000);
 		DOFLAG_PF;
+		SET_FLAG(AF,(lf_var2d&0x1f));
 		break;
 
 
@@ -712,22 +733,28 @@ Bitu FillFlags(void) {
 		SET_FLAG(CF,(lf_var1b >> (lf_var2b - 1)) & 1);
 		DOFLAG_ZFb;
 		DOFLAG_SFb;
-		SET_FLAG(OF,(lf_resb ^ lf_var1b) & 0x80);
+		if ((lf_var2b&0x1f)==1) SET_FLAG(OF,(lf_var1b >= 0x80));
+		else SET_FLAG(OF,false);
 		DOFLAG_PF;
+		SET_FLAG(AF,(lf_var2b&0x1f));
 		break;
 	case t_SHRw:
 		SET_FLAG(CF,(lf_var1w >> (lf_var2b - 1)) & 1);
 		DOFLAG_ZFw;
 		DOFLAG_SFw;
-		SET_FLAG(OF,(lf_resw ^ lf_var1w) & 0x8000);
+		if ((lf_var2w&0x1f)==1) SET_FLAG(OF,(lf_var1w >= 0x8000));
+		else SET_FLAG(OF,false);
 		DOFLAG_PF;
+		SET_FLAG(AF,(lf_var2w&0x1f));
 		break;
 	case t_SHRd:
 		SET_FLAG(CF,(lf_var1d >> (lf_var2b - 1)) & 1);
 		DOFLAG_ZFd;
 		DOFLAG_SFd;
-		SET_FLAG(OF,(lf_resd ^ lf_var1d) & 0x80000000);
+		if ((lf_var2d&0x1f)==1) SET_FLAG(OF,(lf_var1d >= 0x80000000));
+		else SET_FLAG(OF,false);
 		DOFLAG_PF;
+		SET_FLAG(AF,(lf_var2d&0x1f));
 		break;
 
 	
@@ -753,6 +780,7 @@ Bitu FillFlags(void) {
 		DOFLAG_SFb;
 		SET_FLAG(OF,false);
 		DOFLAG_PF;
+		SET_FLAG(AF,(lf_var2b&0x1f));
 		break;
 	case t_SARw:
 		SET_FLAG(CF,(((Bit16s) lf_var1w) >> (lf_var2b - 1)) & 1);
@@ -760,6 +788,7 @@ Bitu FillFlags(void) {
 		DOFLAG_SFw;
 		SET_FLAG(OF,false);
 		DOFLAG_PF;
+		SET_FLAG(AF,(lf_var2w&0x1f));
 		break;
 	case t_SARd:
 		SET_FLAG(CF,(((Bit32s) lf_var1d) >> (lf_var2b - 1)) & 1);
@@ -767,6 +796,7 @@ Bitu FillFlags(void) {
 		DOFLAG_SFd;
 		SET_FLAG(OF,false);
 		DOFLAG_PF;
+		SET_FLAG(AF,(lf_var2d&0x1f));
 		break;
 
 	case t_INCb:
@@ -792,21 +822,21 @@ Bitu FillFlags(void) {
 		break;
 
 	case t_DECb:
-		SET_FLAG(AF,(lf_resb & 0x0f) == 0);
+		SET_FLAG(AF,(lf_resb & 0x0f) == 0x0f);
 		DOFLAG_ZFb;
 		DOFLAG_SFb;
 		SET_FLAG(OF,(lf_resb == 0x7f));
 		DOFLAG_PF;
 		break;
 	case t_DECw:
-		SET_FLAG(AF,(lf_resw & 0x0f) == 0);
+		SET_FLAG(AF,(lf_resw & 0x0f) == 0x0f);
 		DOFLAG_ZFw;
 		DOFLAG_SFw;
 		SET_FLAG(OF,(lf_resw == 0x7fff));
 		DOFLAG_PF;
 		break;
 	case t_DECd:
-		SET_FLAG(AF,(lf_resd & 0x0f) == 0);
+		SET_FLAG(AF,(lf_resd & 0x0f) == 0x0f);
 		DOFLAG_ZFd;
 		DOFLAG_SFd;
 		SET_FLAG(OF,(lf_resd == 0x7fffffff));
@@ -815,7 +845,7 @@ Bitu FillFlags(void) {
 
 	case t_NEGb:
 		SET_FLAG(CF,(lf_var1b!=0));
-		SET_FLAG(AF,(lf_resb & 0x0f) == 0);
+		SET_FLAG(AF,(lf_resb & 0x0f) != 0);
 		DOFLAG_ZFb;
 		DOFLAG_SFb;
 		SET_FLAG(OF,(lf_var1b == 0x80));
@@ -823,7 +853,7 @@ Bitu FillFlags(void) {
 		break;
 	case t_NEGw:
 		SET_FLAG(CF,(lf_var1w!=0));
-		SET_FLAG(AF,(lf_resw & 0x0f) == 0);
+		SET_FLAG(AF,(lf_resw & 0x0f) != 0);
 		DOFLAG_ZFw;
 		DOFLAG_SFw;
 		SET_FLAG(OF,(lf_var1w == 0x8000));
@@ -831,7 +861,7 @@ Bitu FillFlags(void) {
 		break;
 	case t_NEGd:
 		SET_FLAG(CF,(lf_var1d!=0));
-		SET_FLAG(AF,(lf_resd & 0x0f) == 0);
+		SET_FLAG(AF,(lf_resd & 0x0f) != 0);
 		DOFLAG_ZFd;
 		DOFLAG_SFd;
 		SET_FLAG(OF,(lf_var1d == 0x80000000));
@@ -851,4 +881,308 @@ Bitu FillFlags(void) {
 	return reg_flags;
 }
 
+void FillFlagsNoCFOF(void) {
+	switch (lflags.type) {
+	case t_UNKNOWN:
+		return;
+	case t_ADDb:	
+		DOFLAG_AF;
+		DOFLAG_ZFb;
+		DOFLAG_SFb;
+		DOFLAG_PF;
+		break;
+	case t_ADDw:	
+		DOFLAG_AF;
+		DOFLAG_ZFw;
+		DOFLAG_SFw;
+		DOFLAG_PF;
+		break;
+	case t_ADDd:
+		DOFLAG_AF;
+		DOFLAG_ZFd;
+		DOFLAG_SFd;
+		DOFLAG_PF;
+		break;
+	case t_ADCb:
+		DOFLAG_AF;
+		DOFLAG_ZFb;
+		DOFLAG_SFb;
+		DOFLAG_PF;
+		break;
+	case t_ADCw:
+		DOFLAG_AF;
+		DOFLAG_ZFw;
+		DOFLAG_SFw;
+		DOFLAG_PF;
+		break;
+	case t_ADCd:
+		DOFLAG_AF;
+		DOFLAG_ZFd;
+		DOFLAG_SFd;
+		DOFLAG_PF;
+		break;
+
+
+	case t_SBBb:
+		DOFLAG_AF;
+		DOFLAG_ZFb;
+		DOFLAG_SFb;
+		DOFLAG_PF;
+		break;
+	case t_SBBw:
+		DOFLAG_AF;
+		DOFLAG_ZFw;
+		DOFLAG_SFw;
+		DOFLAG_PF;
+		break;
+	case t_SBBd:
+		DOFLAG_AF;
+		DOFLAG_ZFd;
+		DOFLAG_SFd;
+		DOFLAG_PF;
+		break;
+	
+
+	case t_SUBb:
+	case t_CMPb:
+		DOFLAG_AF;
+		DOFLAG_ZFb;
+		DOFLAG_SFb;
+		DOFLAG_PF;
+		break;
+	case t_SUBw:
+	case t_CMPw:
+		DOFLAG_AF;
+		DOFLAG_ZFw;
+		DOFLAG_SFw;
+		DOFLAG_PF;
+		break;
+	case t_SUBd:
+	case t_CMPd:
+		DOFLAG_AF;
+		DOFLAG_ZFd;
+		DOFLAG_SFd;
+		DOFLAG_PF;
+		break;
+
+
+	case t_ORb:
+		SET_FLAG(AF,false);
+		DOFLAG_ZFb;
+		DOFLAG_SFb;
+		DOFLAG_PF;
+		break;
+	case t_ORw:
+		SET_FLAG(AF,false);
+		DOFLAG_ZFw;
+		DOFLAG_SFw;
+		DOFLAG_PF;
+		break;
+	case t_ORd:
+		SET_FLAG(AF,false);
+		DOFLAG_ZFd;
+		DOFLAG_SFd;
+		DOFLAG_PF;
+		break;
+	
+	
+	case t_TESTb:
+	case t_ANDb:
+		SET_FLAG(AF,false);
+		DOFLAG_ZFb;
+		DOFLAG_SFb;
+		DOFLAG_PF;
+		break;
+	case t_TESTw:
+	case t_ANDw:
+		SET_FLAG(AF,false);
+		DOFLAG_ZFw;
+		DOFLAG_SFw;
+		DOFLAG_PF;
+		break;
+	case t_TESTd:
+	case t_ANDd:
+		SET_FLAG(AF,false);
+		DOFLAG_ZFd;
+		DOFLAG_SFd;
+		DOFLAG_PF;
+		break;
+
+	
+	case t_XORb:
+		SET_FLAG(AF,false);
+		DOFLAG_ZFb;
+		DOFLAG_SFb;
+		DOFLAG_PF;
+		break;
+	case t_XORw:
+		SET_FLAG(AF,false);
+		DOFLAG_ZFw;
+		DOFLAG_SFw;
+		DOFLAG_PF;
+		break;
+	case t_XORd:
+		SET_FLAG(AF,false);
+		DOFLAG_ZFd;
+		DOFLAG_SFd;
+		DOFLAG_PF;
+		break;
+
+
+	case t_SHLb:
+		DOFLAG_ZFb;
+		DOFLAG_SFb;
+		DOFLAG_PF;
+		SET_FLAG(AF,(lf_var2b&0x1f));
+		break;
+	case t_SHLw:
+		DOFLAG_ZFw;
+		DOFLAG_SFw;
+		DOFLAG_PF;
+		SET_FLAG(AF,(lf_var2w&0x1f));
+		break;
+	case t_SHLd:
+		DOFLAG_ZFd;
+		DOFLAG_SFd;
+		DOFLAG_PF;
+		SET_FLAG(AF,(lf_var2d&0x1f));
+		break;
+
+
+	case t_DSHLw:
+		DOFLAG_ZFw;
+		DOFLAG_SFw;
+		DOFLAG_PF;
+		break;
+	case t_DSHLd:
+		DOFLAG_ZFd;
+		DOFLAG_SFd;
+		DOFLAG_PF;
+		break;
+
+
+	case t_SHRb:
+		DOFLAG_ZFb;
+		DOFLAG_SFb;
+		DOFLAG_PF;
+		SET_FLAG(AF,(lf_var2b&0x1f));
+		break;
+	case t_SHRw:
+		DOFLAG_ZFw;
+		DOFLAG_SFw;
+		DOFLAG_PF;
+		SET_FLAG(AF,(lf_var2w&0x1f));
+		break;
+	case t_SHRd:
+		DOFLAG_ZFd;
+		DOFLAG_SFd;
+		DOFLAG_PF;
+		SET_FLAG(AF,(lf_var2d&0x1f));
+		break;
+
+	
+	case t_DSHRw:	/* Hmm this is not correct for shift higher than 16 */
+		DOFLAG_ZFw;
+		DOFLAG_SFw;
+		DOFLAG_PF;
+		break;
+	case t_DSHRd:
+		DOFLAG_ZFd;
+		DOFLAG_SFd;
+		DOFLAG_PF;
+		break;
+
+
+	case t_SARb:
+		DOFLAG_ZFb;
+		DOFLAG_SFb;
+		DOFLAG_PF;
+		SET_FLAG(AF,(lf_var2b&0x1f));
+		break;
+	case t_SARw:
+		DOFLAG_ZFw;
+		DOFLAG_SFw;
+		DOFLAG_PF;
+		SET_FLAG(AF,(lf_var2w&0x1f));
+		break;
+	case t_SARd:
+		DOFLAG_ZFd;
+		DOFLAG_SFd;
+		DOFLAG_PF;
+		SET_FLAG(AF,(lf_var2d&0x1f));
+		break;
+
+	case t_INCb:
+		SET_FLAG(AF,(lf_resb & 0x0f) == 0);
+		DOFLAG_ZFb;
+		DOFLAG_SFb;
+		DOFLAG_PF;
+		break;
+	case t_INCw:
+		SET_FLAG(AF,(lf_resw & 0x0f) == 0);
+		DOFLAG_ZFw;
+		DOFLAG_SFw;
+		DOFLAG_PF;
+		break;
+	case t_INCd:
+		SET_FLAG(AF,(lf_resd & 0x0f) == 0);
+		DOFLAG_ZFd;
+		DOFLAG_SFd;
+		DOFLAG_PF;
+		break;
+
+	case t_DECb:
+		SET_FLAG(AF,(lf_resb & 0x0f) == 0x0f);
+		DOFLAG_ZFb;
+		DOFLAG_SFb;
+		DOFLAG_PF;
+		break;
+	case t_DECw:
+		SET_FLAG(AF,(lf_resw & 0x0f) == 0x0f);
+		DOFLAG_ZFw;
+		DOFLAG_SFw;
+		DOFLAG_PF;
+		break;
+	case t_DECd:
+		SET_FLAG(AF,(lf_resd & 0x0f) == 0x0f);
+		DOFLAG_ZFd;
+		DOFLAG_SFd;
+		DOFLAG_PF;
+		break;
+
+	case t_NEGb:
+		SET_FLAG(AF,(lf_resb & 0x0f) != 0);
+		DOFLAG_ZFb;
+		DOFLAG_SFb;
+		DOFLAG_PF;
+		break;
+	case t_NEGw:
+		SET_FLAG(AF,(lf_resw & 0x0f) != 0);
+		DOFLAG_ZFw;
+		DOFLAG_SFw;
+		DOFLAG_PF;
+		break;
+	case t_NEGd:
+		SET_FLAG(AF,(lf_resd & 0x0f) != 0);
+		DOFLAG_ZFd;
+		DOFLAG_SFd;
+		DOFLAG_PF;
+		break;
+
+	
+	case t_DIV:
+	case t_MUL:
+		break;
+
+	default:
+		LOG(LOG_CPU,LOG_ERROR)("Unhandled flag type %d",lflags.type);
+		break;
+	}
+	lflags.type=t_UNKNOWN;
+}
+
+void DestroyConditionFlags(void) {
+	lflags.type=t_UNKNOWN;
+}
+
 #endif
