@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2007  The DOSBox Team
+ *  Copyright (C) 2002-2009  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: debug.cpp,v 1.88 2007/06/12 20:22:07 c2woody Exp $ */
+/* $Id: debug.cpp,v 1.97 2009/04/11 19:49:52 c2woody Exp $ */
 
 #include "dosbox.h"
 #if C_DEBUG
@@ -48,6 +48,7 @@ using namespace std;
 #include "debug_inc.h"
 #include "../cpu/lazyflags.h"
 #include "keyboard.h"
+#include "setup.h"
 
 #ifdef WIN32
 void WIN32_Console();
@@ -55,8 +56,8 @@ void WIN32_Console();
 #include <termios.h>
 #include <unistd.h>
 static struct termios consolesettings;
-int old_cursor_state;
 #endif
+int old_cursor_state;
 
 // Forwards
 static void DrawCode(void);
@@ -76,17 +77,17 @@ Bit32u GetHexValue(char* str, char*& hex);
 
 class DebugPageHandler : public PageHandler {
 public:
-	Bitu readb(PhysPt addr) {
+	Bitu readb(PhysPt /*addr*/) {
 	}
-	Bitu readw(PhysPt addr) {
+	Bitu readw(PhysPt /*addr*/) {
 	}
-	Bitu readd(PhysPt addr) {
+	Bitu readd(PhysPt /*addr*/) {
 	}
-	void writeb(PhysPt addr,Bitu val) {
+	void writeb(PhysPt /*addr*/,Bitu /*val*/) {
 	}
-	void writew(PhysPt addr,Bitu val) {
+	void writew(PhysPt /*addr*/,Bitu /*val*/) {
 	}
-	void writed(PhysPt addr,Bitu val) {
+	void writed(PhysPt /*addr*/,Bitu /*val*/) {
 	}
 
 
@@ -172,6 +173,7 @@ Bit32u GetAddress(Bit16u seg, Bit32u offset)
 	return (seg<<4)+offset;
 }
 
+static char empty_sel[] = { ' ',' ',0 };
 
 bool GetDescriptorInfo(char* selname, char* out1, char* out2)
 {
@@ -186,7 +188,7 @@ bool GetDescriptorInfo(char* selname, char* out1, char* out2)
 	else if (strstr(selname,"ss") || strstr(selname,"SS")) sel = SegValue(ss);
 	else {
 		sel = GetHexValue(selname,selname);
-		if (*selname==0) selname="  ";
+		if (*selname==0) selname=empty_sel;
 	}
 	if (cpu.gdt.GetDescriptor(sel,desc)) {
 		switch (desc.Type()) {
@@ -448,7 +450,8 @@ bool CBreakpoint::CheckBreakpoint(Bitu seg, Bitu off)
 				Bitu address; 
 				if (bp->GetType()==BKPNT_MEMORY_LINEAR) address = bp->GetOffset();
 				else address = GetAddress(bp->GetSegment(),bp->GetOffset());
-				Bit8u value = mem_readb(address);
+				Bit8u value=0;
+				if (mem_readb_checked(address,&value)) return false;
 				if (bp->GetValue() != value) {
 					// Yup, memory value changed
 					DEBUG_ShowMsg("DEBUG: Memory breakpoint %s: %04X:%04X - %02X -> %02X\n",(bp->GetType()==BKPNT_MEMORY_PROT)?"(Prot)":"",bp->GetSegment(),bp->GetOffset(),bp->GetValue(),value);
@@ -592,18 +595,17 @@ void CBreakpoint::ShowList(void)
 		CBreakpoint* bp = (*i);
 		if (bp->GetType()==BKPNT_PHYSICAL) {
 			DEBUG_ShowMsg("%02X. BP %04X:%04X\n",nr,bp->GetSegment(),bp->GetOffset());
-			nr++;
 		} else if (bp->GetType()==BKPNT_INTERRUPT) {
 			if (bp->GetValue()==BPINT_ALL)	DEBUG_ShowMsg("%02X. BPINT %02X\n",nr,bp->GetIntNr());					
 			else							DEBUG_ShowMsg("%02X. BPINT %02X AH=%02X\n",nr,bp->GetIntNr(),bp->GetValue());
-			nr++;
 		} else if (bp->GetType()==BKPNT_MEMORY) {
 			DEBUG_ShowMsg("%02X. BPMEM %04X:%04X (%02X)\n",nr,bp->GetSegment(),bp->GetOffset(),bp->GetValue());
-			nr++;
 		} else if (bp->GetType()==BKPNT_MEMORY_PROT) {
 			DEBUG_ShowMsg("%02X. BPPM %04X:%08X (%02X)\n",nr,bp->GetSegment(),bp->GetOffset(),bp->GetValue());
-			nr++;
+		} else if (bp->GetType()==BKPNT_MEMORY_LINEAR ) {
+			DEBUG_ShowMsg("%02X. BPLM %08X (%02X)\n",nr,bp->GetOffset(),bp->GetValue());
 		};
+		nr++;
 	}
 };
 
@@ -674,9 +676,7 @@ static void DrawData(void) {
 		else mvwprintw (dbg.win_data,1+y,0,"%04X:%08X ",dataSeg,add);
 		for (int x=0; x<16; x++) {
 			address = GetAddress(dataSeg,add);
-			if (!(paging.tlb.handler[address >> 12]->flags & PFLAG_INIT)) {
-				ch = mem_readb(address);
-			} else ch = 0;
+			if (mem_readb_checked(address,&ch)) ch=0;
 			mvwprintw (dbg.win_data,1+y,14+3*x,"%02X",ch);
 			if (ch<32 || !isprint(*reinterpret_cast<unsigned char*>(&ch))) ch='.';
 			mvwprintw (dbg.win_data,1+y,63+x,"%c",ch);
@@ -759,8 +759,7 @@ static void DrawRegisters(void) {
 	wrefresh(dbg.win_reg);
 };
 
-static void DrawCode(void) 
-{
+static void DrawCode(void) {
 	bool saveSel; 
 	Bit32u disEIP = codeViewData.useEIP;
 	PhysPt start  = GetAddress(codeViewData.useCS,codeViewData.useEIP);
@@ -796,16 +795,21 @@ static void DrawCode(void)
 		mvwprintw(dbg.win_code,i,0,"%04X:%04X  ",codeViewData.useCS,disEIP);
 		
 		if (drawsize>10) { toolarge = true; drawsize = 9; };
-		for (c=0;c<drawsize;c++) wprintw(dbg.win_code,"%02X",mem_readb(start+c));
+		for (c=0;c<drawsize;c++) {
+			Bit8u value;
+			if (mem_readb_checked(start+c,&value)) value=0;
+			wprintw(dbg.win_code,"%02X",value);
+		}
 		if (toolarge) { waddstr(dbg.win_code,".."); drawsize++; };
 		// Spacepad up to 20 characters
-		if(drawsize && (drawsize < 10)) {
+		if(drawsize && (drawsize < 11)) {
 			line20[20 - drawsize*2] = 0;
 			waddstr(dbg.win_code,line20);
 			line20[20 - drawsize*2] = ' ';
 		} else waddstr(dbg.win_code,line20);
 
-		char* res = "";
+		char empty_res[] = { 0 };
+		char* res = empty_res;
 		if (showExtend) res = AnalyzeInstruction(dline, saveSel);
 		// Spacepad it up to 28 characters
 		size_t dline_len = strlen(dline);
@@ -1011,12 +1015,12 @@ bool ParseCommand(char* str) {
 		};
 		name[12] = 0;
 		if(!name[0]) return false;
-		DEBUG_ShowMsg("DEBUG: Variable list load (%s) : %s.\n",name,(CDebugVar::SaveVars(name)?"ok":"failure"));
+		DEBUG_ShowMsg("DEBUG: Variable list load (%s) : %s.\n",name,(CDebugVar::LoadVars(name)?"ok":"failure"));
 		return true;
 	};
 
 	if (command == "SR") { // Set register value
-		DEBUG_ShowMsg("DEBUG: Set Register success.\n",(ChangeRegister(found)?"success":"failure"));
+		DEBUG_ShowMsg("DEBUG: Set Register %s.\n",(ChangeRegister(found)?"success":"failure"));
 		return true;
 	};
 
@@ -1029,7 +1033,7 @@ bool ParseCommand(char* str) {
 			if (*found) {
 				Bit8u value = (Bit8u)GetHexValue(found,found);
 				if(*found) found++;
-				mem_writeb(GetAddress(seg,ofs+count),value);
+				mem_writeb_checked(GetAddress(seg,ofs+count),value);
 				count++;
 			}
 		};
@@ -1104,7 +1108,7 @@ bool ParseCommand(char* str) {
 			DEBUG_ShowMsg("DEBUG: Breakpoints deleted.\n");
 		} else {		
 			// delete single breakpoint
-			CBreakpoint::DeleteByIndex(bpNr);
+			DEBUG_ShowMsg("DEBUG: Breakpoint deletion %s.\n",(CBreakpoint::DeleteByIndex(bpNr)?"success":"failure"));
 		}
 		return true;
 	};
@@ -1188,6 +1192,7 @@ bool ParseCommand(char* str) {
 		char out1[200],out2[200];
 		GetDescriptorInfo(found,out1,out2);
 		DEBUG_ShowMsg("SelectorInfo %s:\n%s\n%s\n",found,out1,out2);
+		return true;
 	};
 
 	if (command == "GDT") {LogGDT(); return true;}
@@ -1250,9 +1255,10 @@ bool ParseCommand(char* str) {
 		DEBUG_ShowMsg("F9                        - Set/Remove breakpoint.\n");
 		DEBUG_ShowMsg("F10/F11                   - Step over / trace into instruction.\n");
 		DEBUG_ShowMsg("ALT + D/E/S/X/B           - Set data view to DS:SI/ES:DI/SS:SP/DS:DX/ES:BX.\n");
-		DEBUG_ShowMsg("Escape                    - clear input line.");
+		DEBUG_ShowMsg("Escape                    - Clear input line.");
 		DEBUG_ShowMsg("Up/Down                   - Move code view cursor.\n");
 		DEBUG_ShowMsg("Page Up/Down              - Scroll data view.\n");
+		DEBUG_ShowMsg("Home/End                  - Scroll log messages.\n");
 		DEBUG_ShowMsg("BP     [segment]:[offset] - Set breakpoint.\n");
 		DEBUG_ShowMsg("BPINT  [intNr] *          - Set interrupt breakpoint.\n");
 		DEBUG_ShowMsg("BPINT  [intNr] [ah]       - Set interrupt breakpoint with ah.\n");
@@ -1268,7 +1274,7 @@ bool ParseCommand(char* str) {
 #if C_HEAVY_DEBUG
 		DEBUG_ShowMsg("LOG [num]                 - Write cpu log file.\n");
 		DEBUG_ShowMsg("LOGS/LOGL [num]           - Write short/long cpu log file.\n");
-		DEBUG_ShowMsg("HEAVYLOG                  - Enable/Disable automatic cpu when dosbox exits.\n");
+		DEBUG_ShowMsg("HEAVYLOG                  - Enable/Disable automatic cpu log when dosbox exits.\n");
 		DEBUG_ShowMsg("ZEROPROTECT               - Enable/Disable zero code execution detecion.\n");
 #endif
 		DEBUG_ShowMsg("SR [reg] [value]          - Set register value.\n");
@@ -1300,8 +1306,7 @@ bool ParseCommand(char* str) {
 	return false;
 };
 
-char* AnalyzeInstruction(char* inst, bool saveSelector)
-{
+char* AnalyzeInstruction(char* inst, bool saveSelector) {
 	static char result[256];
 	
 	char instu[256];
@@ -1344,7 +1349,7 @@ char* AnalyzeInstruction(char* inst, bool saveSelector)
 				pos++;
 		};
 		Bit32u address = GetAddress(seg,adr);
-		if (!(paging.tlb.handler[address >> 12]->flags & PFLAG_INIT)) {
+		if (!(get_tlb_readhandler(address)->flags & PFLAG_INIT)) {
 			static char outmask[] = "%s:[%04X]=%02X";
 			
 			if (cpu.pmode) outmask[6] = '8';
@@ -1459,12 +1464,15 @@ char* AnalyzeInstruction(char* inst, bool saveSelector)
 					}	break;
 		}
 		if (jmp) {
+			pos = strchr(instu,'$');
+			if (pos) {
 			pos = strchr(instu,'+');
 			if (pos) {
 				strcpy(result,"(down)");
 			} else {
 				strcpy(result,"(up)");
 			}
+			}
 		} else {
 			sprintf(result,"(no jmp)");
 		}
@@ -1692,12 +1700,11 @@ static void LogGDT(void)
 	};
 };
 
-static void LogLDT(void)
-{
+static void LogLDT(void) {
 	char out1[512];
 	Descriptor desc;
 	Bitu ldtSelector = cpu.gdt.SLDT();
-	cpu.gdt.GetDescriptor(ldtSelector,desc);
+	if (!cpu.gdt.GetDescriptor(ldtSelector,desc)) return;
 	Bitu length = desc.GetLimit();
 	PhysPt address = desc.GetBase();
 	PhysPt max	   = address + length;
@@ -1713,21 +1720,20 @@ static void LogLDT(void)
 	};
 };
 
-static void LogIDT(void)
-{
+static void LogIDT(void) {
 	char out1[512];
 	Descriptor desc;
 	Bitu address = 0;
 	while (address<256*8) {
-		cpu.idt.GetDescriptor(address,desc);
-		sprintf(out1,"%04X: sel:%04X off:%02X",address/8,desc.GetSelector(),desc.GetOffset());
-		LOG(LOG_MISC,LOG_ERROR)(out1);
+		if (cpu.idt.GetDescriptor(address,desc)) {
+			sprintf(out1,"%04X: sel:%04X off:%02X",address/8,desc.GetSelector(),desc.GetOffset());
+			LOG(LOG_MISC,LOG_ERROR)(out1);
+		}
 		address+=8;
 	};
 };
 
-void LogPages(char* selname)
-{
+void LogPages(char* selname) {
 	char out1[512];
 	if (paging.enabled) {
 		Bitu sel = GetHexValue(selname,selname);
@@ -1741,7 +1747,9 @@ void LogPages(char* selname)
 					Bitu entry_addr=(table.block.base<<12)+(i & 0x3ff)*4;
 					entry.load=phys_readd(entry_addr);
 					if (entry.block.p) {
-						sprintf(out1,"page %05Xxxx -> %04Xxxx  flags [uw] %x:%x::%x:%x",i,entry.block.base,entry.block.us,table.block.us,entry.block.wr,table.block.wr);
+						sprintf(out1,"page %05Xxxx -> %04Xxxx  flags [uw] %x:%x::%x:%x [d=%x|a=%x]",
+							i,entry.block.base,entry.block.us,table.block.us,
+							entry.block.wr,table.block.wr,entry.block.d,entry.block.a);
 						LOG(LOG_MISC,LOG_ERROR)(out1);
 					}
 				}
@@ -1764,8 +1772,7 @@ void LogPages(char* selname)
 	}
 };
 
-static void LogCPUInfo(void)
-{
+static void LogCPUInfo(void) {
 	char out1[512];
 	sprintf(out1,"cr0:%08X cr2:%08X cr3:%08X  cpl=%x",cpu.cr0,paging.cr2,paging.cr3,cpu.cpl);
 	LOG(LOG_MISC,LOG_ERROR)(out1);
@@ -1778,13 +1785,15 @@ static void LogCPUInfo(void)
 
 	Bitu sel=CPU_STR();
 	Descriptor desc;
-	cpu.gdt.GetDescriptor(sel,desc);
-	sprintf(out1,"TR selector=%04X, base=%08X limit=%08X*%X",sel,desc.GetBase(),desc.GetLimit(),desc.saved.seg.g?0x4000:1);
-	LOG(LOG_MISC,LOG_ERROR)(out1);
+	if (cpu.gdt.GetDescriptor(sel,desc)) {
+		sprintf(out1,"TR selector=%04X, base=%08X limit=%08X*%X",sel,desc.GetBase(),desc.GetLimit(),desc.saved.seg.g?0x4000:1);
+		LOG(LOG_MISC,LOG_ERROR)(out1);
+	}
 	sel=CPU_SLDT();
-	cpu.gdt.GetDescriptor(sel,desc);
-	sprintf(out1,"LDT selector=%04X, base=%08X limit=%08X*%X",sel,desc.GetBase(),desc.GetLimit(),desc.saved.seg.g?0x4000:1);
-	LOG(LOG_MISC,LOG_ERROR)(out1);
+	if (cpu.gdt.GetDescriptor(sel,desc)) {
+		sprintf(out1,"LDT selector=%04X, base=%08X limit=%08X*%X",sel,desc.GetBase(),desc.GetLimit(),desc.saved.seg.g?0x4000:1);
+		LOG(LOG_MISC,LOG_ERROR)(out1);
+	}
 };
 
 #if C_HEAVY_DEBUG
@@ -1813,7 +1822,9 @@ static void LogInstruction(Bit16u segValue, Bit32u eipValue,  ofstream& out) {
 	} else if (cpuLogType == 2) {
 		char ibytes[200]="";	char tmpc[200];
 		for (Bitu i=0; i<size; i++) {
-			sprintf(tmpc,"%02X ",mem_readb(start+i));
+			Bit8u value;
+			if (mem_readb_checked(start+i,&value)) sprintf(tmpc,"?? ",value);
+			else sprintf(tmpc,"%02X ",value);
 			strcat(ibytes,tmpc);
 		}
 		len = strlen(ibytes);
@@ -1828,13 +1839,13 @@ static void LogInstruction(Bit16u segValue, Bit32u eipValue,  ofstream& out) {
 	    << " DS:"  << setw(4) << SegValue(ds)<< " ES:"  << setw(4) << SegValue(es);
 
 	if(cpuLogType == 0) {
-		out << " SS:"  << setw(4) << SegValue(ss) << " C"  << get_CF()  << " Z"   << get_ZF()  
-		    << " S" << get_SF() << " O"  << get_OF() << " I"  << GETFLAGBOOL(IF);
+		out << " SS:"  << setw(4) << SegValue(ss) << " C"  << (get_CF()>0)  << " Z"   << (get_ZF()>0)  
+		    << " S" << (get_SF()>0) << " O"  << (get_OF()>0) << " I"  << GETFLAGBOOL(IF);
 	} else {
 		out << " FS:"  << setw(4) << SegValue(fs) << " GS:"  << setw(4) << SegValue(gs)
 		    << " SS:"  << setw(4) << SegValue(ss)
-		    << " CF:"  << get_CF()  << " ZF:"   << get_ZF()  << " SF:"  << get_SF()
-		    << " OF:"  << get_OF()  << " AF:"   << get_AF()  << " PF:"  << get_PF()
+		    << " CF:"  << (get_CF()>0)  << " ZF:"   << (get_ZF()>0)  << " SF:"  << (get_SF()>0)
+		    << " OF:"  << (get_OF()>0)  << " AF:"   << (get_AF()>0)  << " PF:"  << (get_PF()>0)
 		    << " IF:"  << GETFLAGBOOL(IF);
 	}
 	if(cpuLogType == 2) {
@@ -1856,8 +1867,14 @@ public:
 
 	void Run(void)
 	{
+		if(cmd->FindExist("/NOMOUSE",false)) {
+	        	real_writed(0,0x33<<2,0);
+			return;
+		}
+	   
 		char filename[128];
 		char args[256];
+	
 		cmd->FindCommand(1,temp_line);
 		safe_strncpy(filename,temp_line.c_str(),128);
 		// Read commandline
@@ -1921,8 +1938,7 @@ static void DEBUG_ProgramStart(Program * * make) {
 
 // INIT 
 
-void DEBUG_SetupConsole(void)
-{
+void DEBUG_SetupConsole(void) {
 	#ifdef WIN32
 	WIN32_Console();
 	#else
@@ -1936,20 +1952,19 @@ void DEBUG_SetupConsole(void)
 	input_count=0;
 	/* Start the Debug Gui */
 	DBGUI_StartUp();
-};
+}
 
-static void DEBUG_ShutDown(Section * sec) 
-{
+static void DEBUG_ShutDown(Section * /*sec*/) {
 	CBreakpoint::DeleteAll();
 	CDebugVar::DeleteAll();
-	#ifndef WIN32
 	curs_set(old_cursor_state);
+	#ifndef WIN32
 	tcsetattr(0, TCSANOW,&consolesettings);
 //	printf("\e[0m\e[2J"); //Seems to destroy scrolling
 	printf("\ec");
 	fflush(NULL);
 	#endif
-};
+}
 
 Bitu debugCallback;
 
@@ -2003,9 +2018,10 @@ bool CDebugVar::SaveVars(char* name)
 {
 	FILE* f = fopen(name,"wb+");
 	if (!f) return false;
+	if (varList.size()>65535) return false;
 
 	// write number of vars
-	Bit16u num = varList.size();
+	Bit16u num = (Bit16u)varList.size();
 	fwrite(&num,1,sizeof(num),f);
 
 	std::list<CDebugVar*>::iterator i;
@@ -2045,8 +2061,7 @@ bool CDebugVar::LoadVars(char* name)
 	return true;
 };
 
-static void SaveMemory(Bitu seg, Bitu ofs1, Bit32u num)
-{
+static void SaveMemory(Bitu seg, Bitu ofs1, Bit32u num) {
 	FILE* f = fopen("MEMDUMP.TXT","wt");
 	if (!f) {
 		DEBUG_ShowMsg("DEBUG: Memory dump failed.\n");
@@ -2056,21 +2071,32 @@ static void SaveMemory(Bitu seg, Bitu ofs1, Bit32u num)
 	char buffer[128];
 	char temp[16];
 
-	while(num>0) {
-
+	while (num>16) {
 		sprintf(buffer,"%04X:%04X   ",seg,ofs1);
 		for (Bit16u x=0; x<16; x++) {
-			sprintf	(temp,"%02X ",mem_readb(GetAddress(seg,ofs1+x)));
-			strcat	(buffer,temp);
-		};
+			Bit8u value;
+			if (mem_readb_checked(GetAddress(seg,ofs1+x),&value)) sprintf(temp,"?? ",value);
+			else sprintf(temp,"%02X ",value);
+			strcat(buffer,temp);
+		}
 		ofs1+=16;
 		num-=16;
 
 		fprintf(f,"%s\n",buffer);
-	};
+	}
+	if (num>0) {
+		sprintf(buffer,"%04X:%04X   ",seg,ofs1);
+		for (Bit16u x=0; x<num; x++) {
+			Bit8u value;
+			if (mem_readb_checked(GetAddress(seg,ofs1+x),&value)) sprintf(temp,"?? ",value);
+			else sprintf(temp,"%02X ",value);
+			strcat(buffer,temp);
+		}
+		fprintf(f,"%s\n",buffer);
+	}
 	fclose(f);
 	DEBUG_ShowMsg("DEBUG: Memory dump success.\n");
-};
+}
 
 static void SaveMemoryBin(Bitu seg, Bitu ofs1, Bit32u num) {
 	FILE* f = fopen("MEMDUMP.BIN","wb");
@@ -2079,17 +2105,17 @@ static void SaveMemoryBin(Bitu seg, Bitu ofs1, Bit32u num) {
 		return;
 	}
 
-	for(Bitu x = 0; x < num;x++) {
-		Bit8u val = mem_readb(GetAddress(seg,ofs1+x));
+	for (Bitu x = 0; x < num;x++) {
+		Bit8u val;
+		if (mem_readb_checked(GetAddress(seg,ofs1+x),&val)) val=0;
 		fwrite(&val,1,1,f);
-	};
+	}
 
 	fclose(f);
 	DEBUG_ShowMsg("DEBUG: Memory dump binary success.\n");
-};
+}
 
-static void OutputVecTable(char* filename)
-{
+static void OutputVecTable(char* filename) {
 	FILE* f = fopen(filename, "wt");
 	if (!f)
 	{
@@ -2105,8 +2131,9 @@ static void OutputVecTable(char* filename)
 }
 
 #define DEBUG_VAR_BUF_LEN 16
-static void DrawVariables(void)
-{
+static void DrawVariables(void) {
+	if (CDebugVar::varList.empty()) return;
+
 	std::list<CDebugVar*>::iterator i;
 	CDebugVar *dv;
 	char buffer[DEBUG_VAR_BUF_LEN];
@@ -2121,8 +2148,11 @@ static void DrawVariables(void)
 
 		dv = static_cast<CDebugVar*>(*i);
 
-		Bit16u value = mem_readw(dv->GetAdr());
-		snprintf(buffer,DEBUG_VAR_BUF_LEN, "0x%04x", value);
+		Bit16u value;
+		if (mem_readw_checked(dv->GetAdr(),&value))
+			snprintf(buffer,DEBUG_VAR_BUF_LEN, "??????", value);
+		else
+			snprintf(buffer,DEBUG_VAR_BUF_LEN, "0x%04x", value);
 
 		int y = idx / 3;
 		int x = (idx % 3) * 26;
@@ -2276,19 +2306,25 @@ bool DEBUG_HeavyIsBreakpoint(void) {
 	}
 	// LogInstruction
 	if (logHeavy) DEBUG_HeavyLogInstruction();
-	if(zeroProtect && mem_readd(SegPhys(cs) + reg_eip) == 0) zero_count++; else zero_count = 0;
-	if(zeroProtect && GCC_UNLIKELY(zero_count == 10)) E_Exit("running zeroed code");
+	if (zeroProtect) {
+		Bit32u value=0;
+		if (!mem_readd_checked(SegPhys(cs)+reg_eip,&value)) {
+			if (value == 0) zero_count++;
+			else zero_count = 0;
+		}
+		if (GCC_UNLIKELY(zero_count == 10)) E_Exit("running zeroed code");
+	}
 
 	if (skipFirstInstruction) {
 		skipFirstInstruction = false;
 		return false;
 	}
-	PhysPt where = SegPhys(cs)+reg_eip;
 	if (CBreakpoint::CheckBreakpoint(SegValue(cs),reg_eip)) {
 		return true;	
 	}
 	return false;
-};
+}
+
 #endif // HEAVY DEBUG
 
 
