@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2007  The DOSBox Team
+ *  Copyright (C) 2002-2009  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: dos_ioctl.cpp,v 1.29 2007/01/08 19:45:39 qbix79 Exp $ */
+/* $Id: dos_ioctl.cpp,v 1.35 2009/04/16 12:16:52 qbix79 Exp $ */
 
 #include <string.h>
 #include "dosbox.h"
@@ -27,7 +27,8 @@
 
 bool DOS_IOCTL(void) {
 	Bitu handle=0;Bit8u drive=0;
-	if (reg_al<8) {				/* call 0-7 use a file handle */
+	/* calls 0-4,6,7,10,12,16 use a file handle */
+	if ((reg_al<4) || (reg_al==0x06) || (reg_al==0x07) || (reg_al==0x0a) || (reg_al==0x0c) || (reg_al==0x10)) {
 		handle=RealHandle(reg_bx);
 		if (handle>=DOS_FILES) {
 			DOS_SetError(DOSERR_INVALID_HANDLE);
@@ -37,12 +38,18 @@ bool DOS_IOCTL(void) {
 			DOS_SetError(DOSERR_INVALID_HANDLE);
 			return false;
 		}
-	} else  { 				/* the others use a diskdrive */
-		drive=reg_bl;if (!drive) drive=dos.current_drive;else drive--;
-		if( !(( drive < DOS_DRIVES ) && Drives[drive]) ) {
-			DOS_SetError(DOSERR_INVALID_DRIVE);
-			return false;
+	} else if (reg_al<0x12) { 				/* those use a diskdrive except 0x0b */
+		if (reg_al!=0x0b) {
+			drive=reg_bl;if (!drive) drive = DOS_GetDefaultDrive();else drive--;
+			if( !(( drive < DOS_DRIVES ) && Drives[drive]) ) {
+				DOS_SetError(DOSERR_INVALID_DRIVE);
+				return false;
+			}
 		}
+	} else {
+		LOG(LOG_DOSMISC,LOG_ERROR)("DOS:IOCTL Call %2X unhandled",reg_al);
+		DOS_SetError(DOSERR_FUNCTION_NUMBER_INVALID);
+		return false;
 	}
 	switch(reg_al) {
 	case 0x00:		/* Get Device Information */
@@ -59,6 +66,19 @@ bool DOS_IOCTL(void) {
 		}
 		reg_ax=reg_dx; //Destroyed officially
 		return true;
+	case 0x01:		/* Set Device Information */
+		if (reg_dh != 0) {
+			DOS_SetError(DOSERR_DATA_INVALID);
+			return false;
+		} else {
+			if (Files[handle]->GetInformation() & 0x8000) {	//Check for device
+				reg_al=(Bit8u)(Files[handle]->GetInformation() & 0xff);
+			} else {
+				DOS_SetError(DOSERR_FUNCTION_NUMBER_INVALID);
+				return false;
+			}
+		}
+		return true;
 	case 0x02:		/* Read from Device Control Channel */
 		if (Files[handle]->GetInformation() & 0xc000) {
 			/* is character device with IOCTL support */
@@ -69,7 +89,7 @@ bool DOS_IOCTL(void) {
 				return true;
 			}
 		}
-		DOS_SetError(0x0001);	// invalid function
+		DOS_SetError(DOSERR_FUNCTION_NUMBER_INVALID);
 		return false;
 	case 0x03:		/* Write to Device Control Channel */
 		if (Files[handle]->GetInformation() & 0xc000) {
@@ -81,7 +101,7 @@ bool DOS_IOCTL(void) {
 				return true;
 			}
 		}
-		DOS_SetError(0x0001);	// invalid function
+		DOS_SetError(DOSERR_FUNCTION_NUMBER_INVALID);
 		return false;
 	case 0x06:      /* Get Input Status */
 		if (Files[handle]->GetInformation() & 0x8000) {		//Check for device
@@ -106,18 +126,36 @@ bool DOS_IOCTL(void) {
 		return true;
 	case 0x08:		/* Check if block device removable */
 		/* cdrom drives and drive a&b are removable */
-		if (drive < 2  || Drives[drive]->isRemovable())
-			reg_ax=0;
-		else 	reg_ax=1;
+		if (drive < 2) reg_ax=0;
+		else if (!Drives[drive]->isRemovable()) reg_ax=1;
+		else {
+			DOS_SetError(DOSERR_FUNCTION_NUMBER_INVALID);
+			return false;
+		}
 		return true;
 	case 0x09:		/* Check if block device remote */
-		reg_dx=0;
-		if (Drives[drive]->isRemote()) reg_dx|=(1 << 12);
-		//TODO Set bit 9 on drives that don't support direct I/O
-		reg_al=0;
+		if (Drives[drive]->isRemote()) {
+			reg_dx=0x1000;	// device is remote
+			// undocumented bits always clear
+		} else {
+			reg_dx=0x0802;	// Open/Close supported; 32bit access supported (any use? fixes Fable installer)
+			// undocumented bits from device attribute word
+			// TODO Set bit 9 on drives that don't support direct I/O
+		}
+		reg_ax=0x300;
+		return true;
+	case 0x0B:		/* Set sharing retry count */
+		if (reg_dx==0) {
+			DOS_SetError(DOSERR_FUNCTION_NUMBER_INVALID);
+			return false;
+		}
 		return true;
 	case 0x0D:		/* Generic block device request */
 		{
+			if (Drives[drive]->isRemovable()) {
+				DOS_SetError(DOSERR_FUNCTION_NUMBER_INVALID);
+				return false;
+			}
 			PhysPt ptr	= SegPhys(ds)+reg_dx;
 			switch (reg_cl) {
 			case 0x60:		/* Get Device parameters */
@@ -139,7 +177,8 @@ bool DOS_IOCTL(void) {
 
 					char const* find_ext=strchr(bufin,'.');
 					if (find_ext) {
-						Bitu size=find_ext-bufin;if (size>8) size=8;
+						Bitu size=(Bitu)(find_ext-bufin);
+						if (size>8) size=8;
 						memcpy(buffer,bufin,size);
 						find_ext++;
 						memcpy(buffer+size,find_ext,(strlen(find_ext)>3) ? 3 : strlen(find_ext)); 
@@ -158,19 +197,25 @@ bool DOS_IOCTL(void) {
 				break;
 			default	:	
 				LOG(LOG_IOCTL,LOG_ERROR)("DOS:IOCTL Call 0D:%2X Drive %2X unhandled",reg_cl,drive);
+				DOS_SetError(DOSERR_FUNCTION_NUMBER_INVALID);
 				return false;
 			}
 			return true;
 		}
-	case 0xE:		/* Get Logical Drive Map */
+	case 0x0E:			/* Get Logical Drive Map */
+		if (Drives[drive]->isRemovable()) {
+			DOS_SetError(DOSERR_FUNCTION_NUMBER_INVALID);
+			return false;
+		}
 		reg_al = 0;		/* Only 1 logical drive assigned */
 		return true;
 	default:
 		LOG(LOG_DOSMISC,LOG_ERROR)("DOS:IOCTL Call %2X unhandled",reg_al);
-		return false;
-	};
+		DOS_SetError(DOSERR_FUNCTION_NUMBER_INVALID);
+		break;
+	}
 	return false;
-};
+}
 
 
 bool DOS_GetSTDINStatus(void) {
@@ -178,4 +223,4 @@ bool DOS_GetSTDINStatus(void) {
 	if (handle==0xFF) return false;
 	if (Files[handle] && (Files[handle]->GetInformation() & 64)) return false;
 	return true;
-};
+}
