@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2007  The DOSBox Team
+ *  Copyright (C) 2002-2009  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -298,7 +298,7 @@ bool DOS_FreeMemory(Bit16u segment) {
 		return false;
 	}
 	mcb.SetPSPSeg(MCB_FREE);
-	DOS_CompressMemory();
+//	DOS_CompressMemory();
 	return true;
 }
 
@@ -386,7 +386,7 @@ bool DOS_LinkUMBsToMemChain(Bit16u linkstate) {
 static Bitu DOS_default_handler(void) {
 	LOG(LOG_CPU,LOG_ERROR)("DOS rerouted Interrupt Called %X",lastint);
 	return CBRET_NONE;
-};
+}
 
 static	CALLBACK_HandlerObject callbackhandler;
 void DOS_SetupMemory(void) {
