@@ -1,6 +1,5 @@
-
 /*
- *  Copyright (C) 2002-2007  The DOSBox Team
+ *  Copyright (C) 2002-2009  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,12 +16,12 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: drive_cache.cpp,v 1.50 2007/06/12 20:22:08 c2woody Exp $ */
+/* $Id: drive_cache.cpp,v 1.59 2009/04/16 12:28:30 qbix79 Exp $ */
 
 #include "drives.h"
 #include "dos_inc.h"
-#include "dirent.h"
 #include "support.h"
+#include "cross.h"
 
 // STL stuff
 #include <vector>
@@ -42,32 +41,27 @@
 
 int fileInfoCounter = 0;
 
-bool SortByName(DOS_Drive_Cache::CFileInfo* const &a, DOS_Drive_Cache::CFileInfo* const &b) 
-{
+bool SortByName(DOS_Drive_Cache::CFileInfo* const &a, DOS_Drive_Cache::CFileInfo* const &b) {
 	return strcmp(a->shortname,b->shortname)<0;
-};
+}
 
-bool SortByNameRev(DOS_Drive_Cache::CFileInfo* const &a, DOS_Drive_Cache::CFileInfo* const &b) 
-{
+bool SortByNameRev(DOS_Drive_Cache::CFileInfo* const &a, DOS_Drive_Cache::CFileInfo* const &b) {
 	return strcmp(a->shortname,b->shortname)>0;
-};
+}
 
-bool SortByDirName(DOS_Drive_Cache::CFileInfo* const &a, DOS_Drive_Cache::CFileInfo* const &b) 
-{
+bool SortByDirName(DOS_Drive_Cache::CFileInfo* const &a, DOS_Drive_Cache::CFileInfo* const &b) {
 	// Directories first...
 	if (a->isDir!=b->isDir) return (a->isDir>b->isDir);	
 	return strcmp(a->shortname,b->shortname)<0;
-};
+}
 
-bool SortByDirNameRev(DOS_Drive_Cache::CFileInfo* const &a, DOS_Drive_Cache::CFileInfo* const &b) 
-{
+bool SortByDirNameRev(DOS_Drive_Cache::CFileInfo* const &a, DOS_Drive_Cache::CFileInfo* const &b) {
 	// Directories first...
 	if (a->isDir!=b->isDir) return (a->isDir>b->isDir);	
 	return strcmp(a->shortname,b->shortname)>0;
-};
+}
 
-DOS_Drive_Cache::DOS_Drive_Cache(void)
-{
+DOS_Drive_Cache::DOS_Drive_Cache(void) {
 	dirBase			= new CFileInfo;
 	save_dir		= 0;
 	srchNr			= 0;
@@ -76,10 +70,9 @@ DOS_Drive_Cache::DOS_Drive_Cache(void)
 	for (Bit32u i=0; i<MAX_OPENDIRS; i++) { dirSearch[i] = 0; free[i] = true; dirFindFirst[i] = 0; };
 	SetDirSort(DIRALPHABETICAL);
 	updatelabel = true;
-};
+}
 
-DOS_Drive_Cache::DOS_Drive_Cache(const char* path)
-{
+DOS_Drive_Cache::DOS_Drive_Cache(const char* path) {
 	dirBase			= new CFileInfo;
 	save_dir		= 0;
 	srchNr			= 0;
@@ -89,23 +82,20 @@ DOS_Drive_Cache::DOS_Drive_Cache(const char* path)
 	SetDirSort(DIRALPHABETICAL);
 	SetBaseDir(path);
 	updatelabel = true;
-};
+}
 
-DOS_Drive_Cache::~DOS_Drive_Cache(void)
-{
+DOS_Drive_Cache::~DOS_Drive_Cache(void) {
 	Clear();
 	for (Bit32u i=0; i<MAX_OPENDIRS; i++) { delete dirFindFirst[i]; dirFindFirst[i]=0; };
-};
+}
 
-void DOS_Drive_Cache::Clear(void)
-{
+void DOS_Drive_Cache::Clear(void) {
 	delete dirBase; dirBase = 0;
 	nextFreeFindFirst	= 0;
 	for (Bit32u i=0; i<MAX_OPENDIRS; i++) dirSearch[i] = 0;
-};
+}
 
-void DOS_Drive_Cache::EmptyCache(void)
-{
+void DOS_Drive_Cache::EmptyCache(void) {
 	// Empty Cache and reinit
 	Clear();
 	dirBase		= new CFileInfo;
@@ -113,47 +103,26 @@ void DOS_Drive_Cache::EmptyCache(void)
 	srchNr		= 0;
 	for (Bit32u i=0; i<MAX_OPENDIRS; i++) free[i] = true; 
 	SetBaseDir(basePath);
-};
+}
 
-void DOS_Drive_Cache::SetLabel(const char* vname,bool allowupdate)
-{
+void DOS_Drive_Cache::SetLabel(const char* vname,bool cdrom,bool allowupdate) {
 /* allowupdate defaults to true. if mount sets a label then allowupdate is 
  * false and will this function return at once after the first call.
  * The label will be set at the first call. */
 
 	if(!this->updatelabel) return;
 	this->updatelabel = allowupdate;
-	Bitu togo		= 8;
-	Bitu vnamePos	= 0;
-	Bitu labelPos	= 0;
-	bool point		= false;
-	while (togo>0) {
-		if (vname[vnamePos]==0) break;
-		if (!point && (vname[vnamePos]=='.')) { togo=4; point=true; }
-		label[labelPos] = toupper(vname[vnamePos]);
-		labelPos++; vnamePos++;
-		togo--;
-		if ((togo==0) && !point) { 
-			if (vname[vnamePos]=='.') vnamePos++;
-			label[labelPos]='.'; labelPos++; point=true; togo=3; 
-		}
-	};
-	label[labelPos]=0;
-	//Remove trailing dot.
-	if((labelPos > 0) && (label[labelPos-1] == '.'))
-		label[labelPos-1]=0;
+	Set_Label(vname,label,cdrom);
 	LOG(LOG_DOSMISC,LOG_NORMAL)("DIRCACHE: Set volume label to %s",label);
-};
+}
 
-Bit16u DOS_Drive_Cache::GetFreeID(CFileInfo* dir)
-{
+Bit16u DOS_Drive_Cache::GetFreeID(CFileInfo* dir) {
 	for (Bit16u i=0; i<MAX_OPENDIRS; i++) if (free[i] || (dir==dirSearch[i])) return i;
 	LOG(LOG_FILES,LOG_NORMAL)("DIRCACHE: Too many open directories!");
 	return 0;
-};
+}
 
-void DOS_Drive_Cache::SetBaseDir(const char* baseDir)
-{
+void DOS_Drive_Cache::SetBaseDir(const char* baseDir) {
 	Bit16u id;
 	strcpy(basePath,baseDir);
 	if (OpenDir(baseDir,id)) {
@@ -162,12 +131,16 @@ void DOS_Drive_Cache::SetBaseDir(const char* baseDir)
 	};
 	// Get Volume Label
 #if defined (WIN32) || defined (OS2)
+	bool cdrom = false;
 	char labellocal[256]={ 0 };
 	char drive[4] = "C:\\";
 	drive[0] = basePath[0];
 #if defined (WIN32)
 	if (GetVolumeInformation(drive,labellocal,256,NULL,NULL,NULL,NULL,0)) {
+	UINT test = GetDriveType(drive);
+	if(test == DRIVE_CDROM) cdrom = true;
 #else // OS2
+	//TODO determine wether cdrom or not!
 	FSINFO fsinfo;
 	ULONG drivenumber = drive[0];
 	if (drivenumber > 26) { // drive letter was lowercase
@@ -177,18 +150,16 @@ void DOS_Drive_Cache::SetBaseDir(const char* baseDir)
 	if (rc == NO_ERROR) {
 #endif
 		/* Set label and allow being updated */
-		SetLabel(labellocal,true);
+		SetLabel(labellocal,cdrom,true);
 	}
 #endif
-};
+}
 
-void DOS_Drive_Cache::ExpandName(char* path)
-{
+void DOS_Drive_Cache::ExpandName(char* path) {
 	strcpy(path,GetExpandName(path));
-};
+}
 
-char* DOS_Drive_Cache::GetExpandName(const char* path)
-{
+char* DOS_Drive_Cache::GetExpandName(const char* path) {
 	static char work [CROSS_LEN] = { 0 };
 	char dir [CROSS_LEN]; 
 
@@ -207,7 +178,7 @@ char* DOS_Drive_Cache::GetExpandName(const char* path)
 		strcat(work,dir);
 	}
 
-	if(work && *work) {
+	if (*work) {
 		size_t len = strlen(work);
 #if defined (WIN32) 
 		if((work[len-1] == CROSS_FILESPLIT ) && (len >= 2) && (work[len-2] != ':')) {
@@ -218,10 +189,9 @@ char* DOS_Drive_Cache::GetExpandName(const char* path)
 		}
 	}
 	return work;
-};
+}
 
-void DOS_Drive_Cache::AddEntry(const char* path, bool checkExists)
-{
+void DOS_Drive_Cache::AddEntry(const char* path, bool checkExists) {
 	// Get Last part...
 	char file	[CROSS_LEN];
 	char expand	[CROSS_LEN];
@@ -231,12 +201,13 @@ void DOS_Drive_Cache::AddEntry(const char* path, bool checkExists)
 
 	if (pos) {
 		strcpy(file,pos+1);	
-		// Check if file already exists, then dont add new entry...
-		if (checkExists && (GetLongName(dir,file)>=0)) return;
+		// Check if file already exists, then don't add new entry...
+		if (checkExists) {
+			if (GetLongName(dir,file)>=0) return;
+		}
+
+		CreateEntry(dir,file,false);
 
-		CreateEntry(dir,file);
-		// Sort Lists - filelist has to be alphabetically sorted
-		std::sort(dir->fileList.begin(), dir->fileList.end(), SortByName);
 		Bits index = GetLongName(dir,file);
 		if (index>=0) {
 			Bit32u i;
@@ -244,16 +215,15 @@ void DOS_Drive_Cache::AddEntry(const char* path, bool checkExists)
 			if (dir) for (i=0; i<MAX_OPENDIRS; i++) {
 				if ((dirSearch[i]==dir) && ((Bit32u)index<=dirSearch[i]->nextEntry)) 
 					dirSearch[i]->nextEntry++;
-			}	
-		};
+			}
+		}
 		//		LOG_DEBUG("DIR: Added Entry %s",path);
 	} else {
 //		LOG_DEBUG("DIR: Error: Failed to add %s",path);	
-	};
-};
+	}
+}
 
-void DOS_Drive_Cache::DeleteEntry(const char* path, bool ignoreLastDir)
-{
+void DOS_Drive_Cache::DeleteEntry(const char* path, bool ignoreLastDir) {
 	CacheOut(path,ignoreLastDir);
 	if (dirSearch[srchNr] && (dirSearch[srchNr]->nextEntry>0)) dirSearch[srchNr]->nextEntry--;
 
@@ -267,16 +237,17 @@ void DOS_Drive_Cache::DeleteEntry(const char* path, bool ignoreLastDir)
 				dirSearch[i]->nextEntry--;
 		}	
 	}
-};
+}
 
-void DOS_Drive_Cache::CacheOut(const char* path, bool ignoreLastDir)
-{
+void DOS_Drive_Cache::CacheOut(const char* path, bool ignoreLastDir) {
 	char expand[CROSS_LEN] = { 0 };
 	CFileInfo* dir;
 	
 	if (ignoreLastDir) {
 		char tmp[CROSS_LEN] = { 0 };
-		Bit32s len = strrchr(path,CROSS_FILESPLIT) - path;
+		Bit32s len=0;
+		const char* pos = strrchr(path,CROSS_FILESPLIT);
+		if (pos) len = (Bit32s)(pos - path);
 		if (len>0) { 
 			safe_strncpy(tmp,path,len+1); 
 		} else	{
@@ -297,22 +268,23 @@ void DOS_Drive_Cache::CacheOut(const char* path, bool ignoreLastDir)
 	dir->fileList.clear();
 	dir->longNameList.clear();
 	save_dir = 0;
-};
+}
 
-bool DOS_Drive_Cache::IsCachedIn(CFileInfo* curDir)
-{
+bool DOS_Drive_Cache::IsCachedIn(CFileInfo* curDir) {
 	return (curDir->fileList.size()>0);
-};
+}
 
 
-bool DOS_Drive_Cache::GetShortName(const char* fullname, char* shortname)
-{
+bool DOS_Drive_Cache::GetShortName(const char* fullname, char* shortname) {
 	// Get Dir Info
 	char expand[CROSS_LEN] = {0};
 	CFileInfo* curDir = FindDirInfo(fullname,expand);
 
+	std::vector<CFileInfo*>::size_type filelist_size = curDir->longNameList.size();
+	if (GCC_UNLIKELY(filelist_size<=0)) return false;
+
 	Bits low		= 0;
-	Bits high		= curDir->longNameList.size()-1;
+	Bits high		= (Bits)(filelist_size-1);
 	Bits mid, res;
 
 	while (low<=high) {
@@ -326,10 +298,9 @@ bool DOS_Drive_Cache::GetShortName(const char* fullname, char* shortname)
 		};
 	}
 	return false;
-};
+}
 
-int DOS_Drive_Cache::CompareShortname(const char* compareName, const char* shortName)
-{
+int DOS_Drive_Cache::CompareShortname(const char* compareName, const char* shortName) {
 	char const* cpos = strchr(shortName,'~');
 	if (cpos) {
 /* the following code is replaced as it's not safe when char* is 64 bits */
@@ -358,13 +329,15 @@ int DOS_Drive_Cache::CompareShortname(const char* compareName, const char* short
 		return strncmp(compareName,shortName,compareCount1);
 	}
 	return strcmp(compareName,shortName);
-};
+}
 
-Bit16u DOS_Drive_Cache::CreateShortNameID(CFileInfo* curDir, const char* name)
-{
-	Bits foundNr	= 0;	
+Bitu DOS_Drive_Cache::CreateShortNameID(CFileInfo* curDir, const char* name) {
+	std::vector<CFileInfo*>::size_type filelist_size = curDir->longNameList.size();
+	if (GCC_UNLIKELY(filelist_size<=0)) return 1;	// shortener IDs start with 1
+
+	Bitu foundNr	= 0;	
 	Bits low		= 0;
-	Bits high		= curDir->longNameList.size()-1;
+	Bits high		= (Bits)(filelist_size-1);
 	Bits mid, res;
 
 	while (low<=high) {
@@ -375,7 +348,7 @@ Bit16u DOS_Drive_Cache::CreateShortNameID(CFileInfo* curDir, const char* name)
 		if (res<0)	high = mid-1; 
 		else {
 			// any more same x chars in next entries ?	
-			do {	
+			do {
 				foundNr = curDir->longNameList[mid]->shortNr;
 				mid++;
 			} while((Bitu)mid<curDir->longNameList.size() && (CompareShortname(name,curDir->longNameList[mid]->shortname)==0));
@@ -383,12 +356,11 @@ Bit16u DOS_Drive_Cache::CreateShortNameID(CFileInfo* curDir, const char* name)
 		};
 	}
 	return foundNr+1;
-};
+}
 
-bool DOS_Drive_Cache::RemoveTrailingDot(char* shortname)
+bool DOS_Drive_Cache::RemoveTrailingDot(char* shortname) {
 // remove trailing '.' if no extension is available (Linux compatibility)
-{
-	Bitu len = strlen(shortname);
+	size_t len = strlen(shortname);
 	if (len && (shortname[len-1]=='.')) {
 		if (len==1) return false;
 		if ((len==2) && (shortname[0]=='.')) return false;
@@ -396,15 +368,17 @@ bool DOS_Drive_Cache::RemoveTrailingDot(char* shortname)
 		return true;
 	}	
 	return false;
-};
+}
+
+Bits DOS_Drive_Cache::GetLongName(CFileInfo* curDir, char* shortName) {
+	std::vector<CFileInfo*>::size_type filelist_size = curDir->fileList.size();
+	if (GCC_UNLIKELY(filelist_size<=0)) return -1;
 
-Bits DOS_Drive_Cache::GetLongName(CFileInfo* curDir, char* shortName)
-{
 	// Remove dot, if no extension...
 	RemoveTrailingDot(shortName);
 	// Search long name and return array number of element
 	Bits low	= 0;
-	Bits high = curDir->fileList.size()-1;
+	Bits high	= (Bits)(filelist_size-1);
 	Bits mid,res;
 	while (low<=high) {
 		mid = (low+high)/2;
@@ -418,11 +392,10 @@ Bits DOS_Drive_Cache::GetLongName(CFileInfo* curDir, char* shortName)
 	}
 	// not available
 	return -1;
-};
+}
 
-bool DOS_Drive_Cache::RemoveSpaces(char* str)
+bool DOS_Drive_Cache::RemoveSpaces(char* str) {
 // Removes all spaces
-{
 	char*	curpos	= str;
 	char*	chkpos	= str;
 	while (*chkpos!=0) { 
@@ -430,12 +403,10 @@ bool DOS_Drive_Cache::RemoveSpaces(char* str)
 	}
 	*curpos = 0;
 	return (curpos!=chkpos);
-};
+}
 
-void DOS_Drive_Cache::CreateShortName(CFileInfo* curDir, CFileInfo* info)
-{
+void DOS_Drive_Cache::CreateShortName(CFileInfo* curDir, CFileInfo* info) {
 	Bits	len			= 0;
-	Bits	lenExt		= 0;
 	bool	createShort = false;
 
 	char tmpNameBuffer[CROSS_LEN];
@@ -450,19 +421,17 @@ void DOS_Drive_Cache::CreateShortName(CFileInfo* curDir, CFileInfo* info)
 	// Get Length of filename
 	char* pos = strchr(tmpName,'.');
 	if (pos) {
-		// Get Length of extension
-		lenExt = strlen(pos)-1;
 		// ignore preceding '.' if extension is longer than "3"
-		if (lenExt>3) {
+		if (strlen(pos)>4) {
 			while (*tmpName=='.') tmpName++;
 			createShort = true;
-		};
+		}
 		pos = strchr(tmpName,'.');
-		if (pos)	len = (Bit16u)(pos - tmpName);
-		else		len = strlen(tmpName);
-
-	} else 
-		len = strlen(tmpName);
+		if (pos)	len = (Bits)(pos - tmpName);
+		else		len = (Bits)strlen(tmpName);
+	} else {
+		len = (Bits)strlen(tmpName);
+	}
 
 	// Should shortname version be created ?
 	createShort = createShort || (len>8);
@@ -478,9 +447,10 @@ void DOS_Drive_Cache::CreateShortName(CFileInfo* curDir, CFileInfo* info)
 		info->shortNr = CreateShortNameID(curDir,tmpName);
 		sprintf(buffer,"%d",info->shortNr);
 		// Copy first letters
-		Bit16u tocopy = 0;
-		if (len+strlen(buffer)+1>8)	tocopy = 8 - strlen(buffer) - 1;
-		else						tocopy = len;
+		Bits tocopy = 0;
+		size_t buflen = strlen(buffer);
+		if (len+buflen+1>8)	tocopy = (Bits)(8 - buflen - 1);
+		else				tocopy = len;
 		safe_strncpy(info->shortname,tmpName,tocopy+1);
 		// Copy number
 		strcat(info->shortname,"~");
@@ -492,18 +462,38 @@ void DOS_Drive_Cache::CreateShortName(CFileInfo* curDir, CFileInfo* info)
 			// add extension
 			strncat(info->shortname,pos,4);
 			info->shortname[DOS_NAMELENGTH] = 0;
-		};
-		// Put it in longname list...
-		curDir->longNameList.push_back(info);
-		std::sort(curDir->longNameList.begin(), curDir->longNameList.end(), SortByName);
+		}
+
+		// keep list sorted for CreateShortNameID to work correctly
+		if (curDir->longNameList.size()>0) {
+			if (!(strcmp(info->shortname,curDir->longNameList.back()->shortname)<0)) {
+				// append at end of list
+				curDir->longNameList.push_back(info);
+			} else {
+				// look for position where to insert this element
+				bool found=false;
+				std::vector<CFileInfo*>::iterator it;
+				for (it=curDir->longNameList.begin(); it!=curDir->longNameList.end(); ++it) {
+					if (strcmp(info->shortname,(*it)->shortname)<0) {
+						found = true;
+						break;
+					}
+				}
+				// Put it in longname list...
+				if (found) curDir->longNameList.insert(it,info);
+				else curDir->longNameList.push_back(info);
+			}
+		} else {
+			// empty file list, append
+			curDir->longNameList.push_back(info);
+		}
 	} else {
 		strcpy(info->shortname,tmpName);
 	}
 	RemoveTrailingDot(info->shortname);
-};
+}
 
-DOS_Drive_Cache::CFileInfo* DOS_Drive_Cache::FindDirInfo(const char* path, char* expandedPath)
-{
+DOS_Drive_Cache::CFileInfo* DOS_Drive_Cache::FindDirInfo(const char* path, char* expandedPath) {
 	// statics
 	static char	split[2] = { CROSS_FILESPLIT,0 };
 	
@@ -580,10 +570,9 @@ DOS_Drive_Cache::CFileInfo* DOS_Drive_Cache::FindDirInfo(const char* path, char*
 	save_dir = curDir;
 
 	return curDir;
-};
+}
 
-bool DOS_Drive_Cache::OpenDir(const char* path, Bit16u& id)
-{
+bool DOS_Drive_Cache::OpenDir(const char* path, Bit16u& id) {
 	char expand[CROSS_LEN] = {0};
 	CFileInfo* dir = FindDirInfo(path,expand);
 	if (OpenDir(dir,expand,id)) {
@@ -591,10 +580,9 @@ bool DOS_Drive_Cache::OpenDir(const char* path, Bit16u& id)
 		return true;
 	}
 	return false;
-};	
+}
 
-bool DOS_Drive_Cache::OpenDir(CFileInfo* dir, const char* expand, Bit16u& id)
-{
+bool DOS_Drive_Cache::OpenDir(CFileInfo* dir, const char* expand, Bit16u& id) {
 	id = GetFreeID(dir);
 	dirSearch[id] = dir;
 	char expandcopy [CROSS_LEN];
@@ -605,58 +593,88 @@ bool DOS_Drive_Cache::OpenDir(CFileInfo* dir, const char* expand, Bit16u& id)
 	// open dir
 	if (dirSearch[id]) {
 		// open dir
-		DIR* dirp = opendir(expandcopy);
+		dir_information* dirp = open_directory(expandcopy);
 		if (dirp) { 
 			// Reset it..
-			closedir(dirp);
+			close_directory(dirp);
 			strcpy(dirPath,expandcopy);
 			free[id] = false;
 			return true;
 		}
 	};
 	return false;
-};
+}
 
-void DOS_Drive_Cache::CreateEntry(CFileInfo* dir, const char* name)
-{
-	struct stat status;
+void DOS_Drive_Cache::CreateEntry(CFileInfo* dir, const char* name, bool is_directory) {
 	CFileInfo* info = new CFileInfo;
-	strcpy(info->orgname ,name);				
+	strcpy(info->orgname, name);				
 	info->shortNr = 0;
-	// Read and copy file stats
-	char buffer[CROSS_LEN];
-	strcpy(buffer,dirPath);
-	strcat(buffer,info->orgname);
-	if (stat(buffer,&status)==0)	info->isDir	= (S_ISDIR(status.st_mode)>0);
-	else							info->isDir = false;
+	info->isDir = is_directory;
+
 	// Check for long filenames...
 	CreateShortName(dir, info);		
-	// Put file in lists
-	dir->fileList.push_back(info);
-};
 
-bool DOS_Drive_Cache::ReadDir(Bit16u id, char* &result)
-{
+	bool found = false;
+
+	// keep list sorted (so GetLongName works correctly, used by CreateShortName in this routine)
+	if (dir->fileList.size()>0) {
+		if (!(strcmp(info->shortname,dir->fileList.back()->shortname)<0)) {
+			// append at end of list
+			dir->fileList.push_back(info);
+		} else {
+			// look for position where to insert this element
+			std::vector<CFileInfo*>::iterator it;
+			for (it=dir->fileList.begin(); it!=dir->fileList.end(); ++it) {
+				if (strcmp(info->shortname,(*it)->shortname)<0) {
+					found = true;
+					break;
+				}
+			}
+			// Put file in lists
+			if (found) dir->fileList.insert(it,info);
+			else dir->fileList.push_back(info);
+		}
+	} else {
+		// empty file list, append
+		dir->fileList.push_back(info);
+	}
+}
+
+void DOS_Drive_Cache::CopyEntry(CFileInfo* dir, CFileInfo* from) {
+	CFileInfo* info = new CFileInfo;
+	// just copy things into new fileinfo
+	strcpy(info->orgname, from->orgname);				
+	strcpy(info->shortname, from->shortname);				
+	info->shortNr = from->shortNr;
+	info->isDir = from->isDir;
+
+	dir->fileList.push_back(info);
+}
+
+bool DOS_Drive_Cache::ReadDir(Bit16u id, char* &result) {
 	// shouldnt happen...
 	if (id>MAX_OPENDIRS) return false;
 
 	if (!IsCachedIn(dirSearch[id])) {
 		// Try to open directory
-		DIR* dirp = opendir(dirPath);
+		dir_information* dirp = open_directory(dirPath);
 		if (!dirp) {
 			free[id] = true;
 			return false;
 		}
 		// Read complete directory
-		struct dirent* tmpres;
-		while ((tmpres = readdir(dirp))!=NULL) {			
-			CreateEntry(dirSearch[id],tmpres->d_name);
-			// Sort Lists - filelist has to be alphabetically sorted, even in between (for finding double file names) 
-			// hmpf.. bit slow probably...
-			std::sort(dirSearch[id]->fileList.begin(), dirSearch[id]->fileList.end(), SortByName);
+		char dir_name[CROSS_LEN];
+		bool is_directory;
+		if (read_directory_first(dirp, dir_name, is_directory)) {
+			CreateEntry(dirSearch[id], dir_name, is_directory);
+			while (read_directory_next(dirp, dir_name, is_directory)) {
+				CreateEntry(dirSearch[id], dir_name, is_directory);
+			}
 		}
+
 		// close dir
-		closedir(dirp);
+		close_directory(dirp);
+
 		// Info
 /*		if (!dirp) {
 			LOG_DEBUG("DIR: Error Caching in %s",dirPath);			
@@ -670,9 +688,9 @@ bool DOS_Drive_Cache::ReadDir(Bit16u id, char* &result)
 	if (SetResult(dirSearch[id], result, dirSearch[id]->nextEntry)) return true;
 	free[id] = true;
 	return false;
-};
+}
 
-bool DOS_Drive_Cache::SetResult(CFileInfo* dir, char* &result, Bit16u entryNr)
+bool DOS_Drive_Cache::SetResult(CFileInfo* dir, char* &result, Bitu entryNr)
 {
 	static char res[CROSS_LEN] = { 0 };
 
@@ -684,11 +702,10 @@ bool DOS_Drive_Cache::SetResult(CFileInfo* dir, char* &result, Bit16u entryNr)
 	// Set to next Entry
 	dir->nextEntry = entryNr+1;
 	return true;
-};
+}
 
 // FindFirst / FindNext
-bool DOS_Drive_Cache::FindFirst(char* path, Bitu& id)
-{
+bool DOS_Drive_Cache::FindFirst(char* path, Bitu& id) {
 	Bit16u	dirID;
 	Bitu	dirFindFirstID = this->nextFreeFindFirst;
 
@@ -704,7 +721,7 @@ bool DOS_Drive_Cache::FindFirst(char* path, Bitu& id)
 		dirFindFirstID = 0;
 		this->nextFreeFindFirst = 1; //the next free one after this search
 		for(Bitu n=0; n<MAX_OPENDIRS;n++) {	
-	     		// Clear and reuse slot
+	     	// Clear and reuse slot
 			delete dirFindFirst[n];
 			dirFindFirst[n]=0;
 		}
@@ -715,26 +732,24 @@ bool DOS_Drive_Cache::FindFirst(char* path, Bitu& id)
 
 	// Copy entries to use with FindNext
 	for (Bitu i=0; i<dirSearch[dirID]->fileList.size(); i++) {
-		CreateEntry(dirFindFirst[dirFindFirstID],dirSearch[dirID]->fileList[i]->orgname);
-		// Sort Lists - filelist has to be alphabetically sorted, even in between (for finding double file names) 
-		std::sort(dirFindFirst[dirFindFirstID]->fileList.begin(), dirFindFirst[dirFindFirstID]->fileList.end(), SortByName);
-	};
+		CopyEntry(dirFindFirst[dirFindFirstID],dirSearch[dirID]->fileList[i]);
+	}
 	// Now re-sort the fileList accordingly to output
 	switch (sortDirType) {
-		case ALPHABETICAL		: std::sort(dirFindFirst[dirFindFirstID]->fileList.begin(), dirFindFirst[dirFindFirstID]->fileList.end(), SortByName);		break;
+		case ALPHABETICAL		: break;
+//		case ALPHABETICAL		: std::sort(dirFindFirst[dirFindFirstID]->fileList.begin(), dirFindFirst[dirFindFirstID]->fileList.end(), SortByName);		break;
 		case DIRALPHABETICAL	: std::sort(dirFindFirst[dirFindFirstID]->fileList.begin(), dirFindFirst[dirFindFirstID]->fileList.end(), SortByDirName);		break;
 		case ALPHABETICALREV	: std::sort(dirFindFirst[dirFindFirstID]->fileList.begin(), dirFindFirst[dirFindFirstID]->fileList.end(), SortByNameRev);		break;
 		case DIRALPHABETICALREV	: std::sort(dirFindFirst[dirFindFirstID]->fileList.begin(), dirFindFirst[dirFindFirstID]->fileList.end(), SortByDirNameRev);	break;
 		case NOSORT				: break;
-	};	
+	}
 
 //	LOG(LOG_MISC,LOG_ERROR)("DIRCACHE: FindFirst : %s (ID:%02X)",path,dirFindFirstID);
 	id = dirFindFirstID;
 	return true;
-};
+}
 
-bool DOS_Drive_Cache::FindNext(Bitu id, char* &result)
-{
+bool DOS_Drive_Cache::FindNext(Bitu id, char* &result) {
 	// out of range ?
 	if ((id>=MAX_OPENDIRS) || !dirFindFirst[id]) {
 		LOG(LOG_MISC,LOG_ERROR)("DIRCACHE: FindFirst/Next failure : ID out of range: %04X",id);
@@ -746,4 +761,4 @@ bool DOS_Drive_Cache::FindNext(Bitu id, char* &result)
 		return false;
 	}
 	return true;
-};
+}
