@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2007  The DOSBox Team
+ *  Copyright (C) 2002-2009  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: drive_fat.cpp,v 1.20 2007/01/08 19:45:39 qbix79 Exp $ */
+/* $Id: drive_fat.cpp,v 1.27 2009/05/27 09:15:41 qbix79 Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -300,7 +300,8 @@ Bit32u fatDrive::getClusterValue(Bit32u clustNum) {
 	if(curFatSect != fatsectnum) {
 		/* Load two sectors at once for FAT12 */
 		loadedDisk->Read_AbsoluteSector(fatsectnum, &fatSectBuffer[0]);
-		loadedDisk->Read_AbsoluteSector(fatsectnum+1, &fatSectBuffer[512]);
+		if (fattype==FAT12)
+			loadedDisk->Read_AbsoluteSector(fatsectnum+1, &fatSectBuffer[512]);
 		curFatSect = fatsectnum;
 	}
 
@@ -346,7 +347,8 @@ void fatDrive::setClusterValue(Bit32u clustNum, Bit32u clustValue) {
 	if(curFatSect != fatsectnum) {
 		/* Load two sectors at once for FAT12 */
 		loadedDisk->Read_AbsoluteSector(fatsectnum, &fatSectBuffer[0]);
-		loadedDisk->Read_AbsoluteSector(fatsectnum+1, &fatSectBuffer[512]);
+		if (fattype==FAT12)
+			loadedDisk->Read_AbsoluteSector(fatsectnum+1, &fatSectBuffer[512]);
 		curFatSect = fatsectnum;
 	}
 
@@ -374,10 +376,12 @@ void fatDrive::setClusterValue(Bit32u clustNum, Bit32u clustValue) {
 			*((Bit32u *)&fatSectBuffer[fatentoff]) = clustValue;
 			break;
 	}
-	int fc;
-	for(fc=0;fc<bootbuffer.fatcopies;fc++) {
+	for(int fc=0;fc<bootbuffer.fatcopies;fc++) {
 		loadedDisk->Write_AbsoluteSector(fatsectnum + (fc * bootbuffer.sectorsperfat), &fatSectBuffer[0]);
-		loadedDisk->Write_AbsoluteSector(fatsectnum+1+(fc * bootbuffer.sectorsperfat), &fatSectBuffer[512]);
+		if (fattype==FAT12) {
+			if (fatentoff>=511)
+				loadedDisk->Write_AbsoluteSector(fatsectnum+1+(fc * bootbuffer.sectorsperfat), &fatSectBuffer[512]);
+		}
 	}
 }
 
@@ -390,6 +394,9 @@ bool fatDrive::getEntryName(char *fullname, char *entname) {
 
 	//LOG_MSG("Testing for filename %s", fullname);
 	findDir = strtok(dirtoken,"\\");
+	if (findDir==NULL) {
+		return true;	// root always exists
+	}
 	findFile = findDir;
 	while(findDir != NULL) {
 		findFile = findDir;
@@ -399,8 +406,8 @@ bool fatDrive::getEntryName(char *fullname, char *entname) {
 	return true;
 }
 
-bool fatDrive::getFileDirEntry(char * filename, direntry * useEntry, Bit32u * dirClust, Bit32u * subEntry) {
-	Bit32u len = (Bit32u)strlen(filename);
+bool fatDrive::getFileDirEntry(char const * const filename, direntry * useEntry, Bit32u * dirClust, Bit32u * subEntry) {
+	size_t len = strlen(filename);
 	char dirtoken[DOS_PATHLENGTH];
 	Bit32u currentClust = 0;
 
@@ -408,6 +415,7 @@ bool fatDrive::getFileDirEntry(char * filename, direntry * useEntry, Bit32u * di
 	char * findDir;
 	char * findFile;
 	strcpy(dirtoken,filename);
+	findFile=dirtoken;
 
 	/* Skip if testing in root directory */
 	if ((len>0) && (filename[len-1]!='\\')) {
@@ -759,6 +767,8 @@ bool fatDrive::FileCreate(DOS_File **file, char *name, Bit16u attributes) {
 	char dirName[DOS_NAMELENGTH_ASCII];
 	char pathName[11];
 
+	Bitu save_errorcode=dos.errorcode;
+
 	/* Check if file already exists */
 	if(getFileDirEntry(name, &fileEntry, &dirClust, &subEntry)) {
 		/* Truncate file */
@@ -788,13 +798,15 @@ bool fatDrive::FileCreate(DOS_File **file, char *name, Bit16u attributes) {
 	/* Maybe modTime and date should be used ? (crt matches findnext) */
 	((fatFile *)(*file))->time = fileEntry.crtTime;
 	((fatFile *)(*file))->date = fileEntry.crtDate;
+
+	dos.errorcode=save_errorcode;
 	return true;
 }
 
 bool fatDrive::FileExists(const char *name) {
 	direntry fileEntry;
 	Bit32u dummy1, dummy2;
-	if(!getFileDirEntry((char *)name, &fileEntry, &dummy1, &dummy2)) return false;
+	if(!getFileDirEntry(name, &fileEntry, &dummy1, &dummy2)) return false;
 	return true;
 }
 
@@ -854,25 +866,22 @@ bool fatDrive::FindFirst(char *_dir, DOS_DTA &dta,bool /*fcb_findfirst*/) {
 	return FindNextInternal(cwdDirCluster, dta, &dummyClust);
 }
 
-char* removeTrailingSpaces(char* str)
-{
-    char* end = str + strlen(str);
-    while(*--end == ' ' && end > str);
-    *++end = '\0';
-    return str;
+char* removeTrailingSpaces(char* str) {
+	char* end = str + strlen(str);
+	while((*--end == ' ') && (end > str)) {};
+	*++end = '\0';
+	return str;
 }
 
-char* removeLeadingSpaces(char* str)
-{
-    size_t len = strlen(str);
-    size_t pos = strspn(str," ");
-    memmove(str,str + pos,len - pos + 1);
-    return str;
+char* removeLeadingSpaces(char* str) {
+	size_t len = strlen(str);
+	size_t pos = strspn(str," ");
+	memmove(str,str + pos,len - pos + 1);
+	return str;
 }
 
-char* trimString(char* str)
-{
-    return removeTrailingSpaces(removeLeadingSpaces(str));
+char* trimString(char* str) {
+	return removeTrailingSpaces(removeLeadingSpaces(str));
 }
 
 bool fatDrive::FindNextInternal(Bit32u dirClustNumber, DOS_DTA &dta, direntry *foundEntry) {
@@ -981,9 +990,9 @@ bool fatDrive::directoryBrowse(Bit32u dirClustNumber, direntry *useEntry, Bit32s
 	Bit32u entryoffset = 0;	/* Index offset within sector */
 	Bit32u tmpsector;
 	Bit16u dirPos = 0;
-	
+
 	while(entNum>=0) {
-		
+
 		logentsector = dirPos / 16;
 		entryoffset = dirPos % 16;
 
@@ -1004,7 +1013,7 @@ bool fatDrive::directoryBrowse(Bit32u dirClustNumber, direntry *useEntry, Bit32s
 		if (sectbuf[entryoffset].entryname[0] == 0x00) return false;
 		--entNum;
 	}
-	
+
 	memcpy(useEntry, &sectbuf[entryoffset],sizeof(direntry));
 	return true;
 }
@@ -1205,7 +1214,42 @@ bool fatDrive::RemoveDir(char *dir) {
 	return true;
 }
 
-bool fatDrive::Rename(char * /*oldname*/, char * /*newname*/) {
+bool fatDrive::Rename(char * oldname, char * newname) {
+	direntry fileEntry1;
+	Bit32u dirClust1, subEntry1;
+	if(!getFileDirEntry(oldname, &fileEntry1, &dirClust1, &subEntry1)) return false;
+	/* File to be renamed really exists */
+
+	direntry fileEntry2;
+	Bit32u dirClust2, subEntry2;
+
+	/* Check if file already exists */
+	if(!getFileDirEntry(newname, &fileEntry2, &dirClust2, &subEntry2)) {
+		/* Target doesn't exist, can rename */
+
+		char dirName2[DOS_NAMELENGTH_ASCII];
+		char pathName2[11];
+		/* Can we even get the name of the file itself? */
+		if(!getEntryName(newname, &dirName2[0])) return false;
+		convToDirFile(&dirName2[0], &pathName2[0]);
+
+		/* Can we find the base directory? */
+		if(!getDirClustNum(newname, &dirClust2, true)) return false;
+		memcpy(&fileEntry2, &fileEntry1, sizeof(direntry));
+		memcpy(&fileEntry2.entryname, &pathName2[0], 11);
+		addDirectoryEntry(dirClust2, fileEntry2);
+
+		/* Check if file exists now */
+		if(!getFileDirEntry(newname, &fileEntry2, &dirClust2, &subEntry2)) return false;
+
+		/* Remove old entry */
+		fileEntry1.entryname[0] = 0xe5;
+		directoryChange(dirClust1, &fileEntry1, subEntry1);
+
+		return true;
+	}
+
+	/* Target already exists, fail */
 	return false;
 }
 
