@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2007  The DOSBox Team
+ *  Copyright (C) 2002-2009  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,6 +16,8 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+/* $Id: gus.cpp,v 1.36 2009/04/27 17:11:26 qbix79 Exp $ */
+
 #include <string.h>
 #include <iomanip>
 #include <sstream>
@@ -41,8 +43,6 @@ using namespace std;
 #define RAMP_FRACT (10)
 #define RAMP_FRACT_MASK ((1 << RAMP_FRACT)-1)
 
-#define USEVOLTABLE 1
-
 #define GUS_BASE myGUS.portbase
 #define GUS_RATE myGUS.rate
 #define LOG_GUS 0
@@ -52,10 +52,8 @@ static MixerChannel * gus_chan;
 static Bit8u irqtable[8] = { 0, 2, 5, 3, 7, 11, 12, 15 };
 static Bit8u dmatable[8] = { 0, 1, 3, 5, 6, 7, 0, 0 };
 static Bit8u GUSRam[1024*1024]; // 1024K of GUS Ram
-static Bit32s AutoAmp=512;
-#if USEVOLTABLE
+static Bit32s AutoAmp = 512;
 static Bit16u vol16bit[4096];
-#endif
 static Bit32u pantable[16];
 
 class GUSChannels;
@@ -85,13 +83,12 @@ struct GFGus {
 	} timers[2];
 	Bit32u rate;
 	Bitu portbase;
-	Bitu dma1;
-	Bitu dma2;
+	Bit8u dma1;
+	Bit8u dma2;
 
-	Bitu irq1;
-	Bitu irq2;
+	Bit8u irq1;
+	Bit8u irq2;
 
-	char ultradir[512];
 	bool irqenabled;
 	bool ChangeIRQDMA;
 	// IRQ status register values
@@ -270,12 +267,8 @@ public:
 		templeft&=~(templeft >> 31);
 		Bit32s tempright=RampVol - PanRight;
 		tempright&=~(tempright >> 31);
-#if USEVOLTABLE
 		VolLeft=vol16bit[templeft >> RAMP_FRACT];
 		VolRight=vol16bit[tempright >> RAMP_FRACT];
-#else
-
-#endif
 	}
 	INLINE void RampUpdate(void) {
 		/* Check if ramping enabled */
@@ -376,7 +369,7 @@ static void CheckVoiceIrq(void) {
 	if (myGUS.RampIRQ) myGUS.IRQStatus|=0x40;
 	if (myGUS.WaveIRQ) myGUS.IRQStatus|=0x20;
 	GUS_CheckIRQ();
-	while (1) {
+	for (;;) {
 		Bit32u check=(1 << myGUS.IRQChan);
 		if (totalmask & check) return;
 		myGUS.IRQChan++;
@@ -452,7 +445,7 @@ static void GUS_TimerEvent(Bitu val) {
 	}
 	if (myGUS.timers[val].running) 
 		PIC_AddEvent(GUS_TimerEvent,myGUS.timers[val].delay,val);
-};
+}
 
  
 static void ExecuteGlobRegister(void) {
@@ -770,13 +763,11 @@ static void GUS_CallBack(Bitu len) {
 // Generate logarithmic to linear volume conversion tables
 static void MakeTables(void) {
 	int i;
-#if USEVOLTABLE
 	double out = (double)(1 << 13);
 	for (i=4095;i>=0;i--) {
 		vol16bit[i]=(Bit16s)out;
 		out/=1.002709201;		/* 0.0235 dB Steps */
 	}
-#endif
 	pantable[0]=0;
 	for (i=1;i<16;i++) {
 		pantable[i]=(Bit32u)(-128.0*(log((double)i/15.0)/log(2.0))*(double)(1 << RAMP_FRACT));
@@ -791,7 +782,7 @@ private:
 	MixerObject MixerChan;
 public:
 	GUS(Section* configuration):Module_base(configuration){
-		if(machine!=MCH_VGA) return;
+		if(!IS_EGAVGA_ARCH) return;
 		Section_prop * section=static_cast<Section_prop *>(configuration);
 		if(!section->Get_bool("gus")) return;
 	
@@ -801,12 +792,15 @@ public:
 		myGUS.rate=section->Get_int("gusrate");
 	
 		myGUS.portbase = section->Get_hex("gusbase") - 0x200;
-		myGUS.dma1 = section->Get_int("dma1");
-		myGUS.dma2 = section->Get_int("dma2");
-		myGUS.irq1 = section->Get_int("irq1");
-		myGUS.irq2 = section->Get_int("irq2");
-		strcpy(&myGUS.ultradir[0], section->Get_string("ultradir"));
-	
+		int dma_val = section->Get_int("gusdma");
+		if ((dma_val<0) || (dma_val>255)) dma_val = 3;	// sensible default
+		int irq_val = section->Get_int("gusirq");
+		if ((irq_val<0) || (irq_val>255)) irq_val = 5;	// sensible default
+		myGUS.dma1 = (Bit8u)dma_val;
+		myGUS.dma2 = (Bit8u)dma_val;
+		myGUS.irq1 = (Bit8u)irq_val;
+		myGUS.irq2 = (Bit8u)irq_val;
+
 		// We'll leave the MIDI interface to the MPU-401 
 		// Ditto for the Joystick 
 		// GF1 Synthesizer 
@@ -842,9 +836,8 @@ public:
 	
 		MakeTables();
 	
-		int i;
-		for(i=0;i<32;i++) {
-			guschan[i] = new GUSChannels(i);
+		for (Bit8u chan_ct=0; chan_ct<32; chan_ct++) {
+			guschan[chan_ct] = new GUSChannels(chan_ct);
 		}
 		// Register the Mixer CallBack 
 		gus_chan=MixerChan.Install(GUS_CallBack,GUS_RATE,"GUS");
@@ -852,19 +845,21 @@ public:
 		GUSReset();
 		myGUS.gRegData=0x0;
 		int portat = 0x200+GUS_BASE;
+
 		// ULTRASND=Port,DMA1,DMA2,IRQ1,IRQ2
-		// Create autoexec.bat lines
+		// [GUS port], [GUS DMA (recording)], [GUS DMA (playback)], [GUS IRQ (playback)], [GUS IRQ (MIDI)]
 		ostringstream temp;
 		temp << "SET ULTRASND=" << hex << setw(3) << portat << ","
-		     << dec << myGUS.dma1 << "," << myGUS.dma2 << ","
-		     << myGUS.irq1 << "," << myGUS.irq2 << ends;
+		     << dec << (Bitu)myGUS.dma1 << "," << (Bitu)myGUS.dma2 << ","
+		     << (Bitu)myGUS.irq1 << "," << (Bitu)myGUS.irq2 << ends;
+		// Create autoexec.bat lines
 		autoexecline[0].Install(temp.str());
-		autoexecline[1].Install(std::string("SET ULTRADIR=")+ myGUS.ultradir);
+		autoexecline[1].Install(std::string("SET ULTRADIR=") + section->Get_string("ultradir"));
 	}
 
 
 	~GUS() {
-		if(machine!=MCH_VGA) return;
+		if(!IS_EGAVGA_ARCH) return;
 		Section_prop * section=static_cast<Section_prop *>(m_configuration);
 		if(!section->Get_bool("gus")) return;
 	
@@ -883,7 +878,7 @@ public:
 
 static GUS* test;
 
-void GUS_ShutDown(Section* sec) {
+void GUS_ShutDown(Section* /*sec*/) {
 	delete test;	
 }
 
