@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2007  The DOSBox Team
+ *  Copyright (C) 2002-2009  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* $Id: mouse.cpp,v 1.69 2007/06/12 20:22:09 c2woody Exp $ */
+/* $Id: mouse.cpp,v 1.79 2009/04/16 12:11:45 qbix79 Exp $ */
 
 #include <string.h>
 #include <math.h>
@@ -32,20 +32,20 @@
 #include "inout.h"
 #include "int10.h"
 #include "bios.h"
-
+#include "dos_inc.h"
 
 static Bitu call_int33,call_int74,int74_ret_callback,call_mouse_bd;
 static Bit16u ps2cbseg,ps2cbofs;
 static bool useps2callback,ps2callbackinit;
-static Bit16u call_ps2;
+static Bitu call_ps2;
 static RealPt ps2_callback;
 static Bit16s oldmouseX, oldmouseY;
 // forward
 void WriteMouseIntVector(void);
 
 struct button_event {
-	Bit16u type;
-	Bit16u buttons;
+	Bit8u type;
+	Bit8u buttons;
 };
 
 #define QUEUE_SIZE 32
@@ -79,20 +79,20 @@ static Bit16u userdefScreenMask[CURSORY];
 static Bit16u userdefCursorMask[CURSORY];
 
 static struct {
-	Bit16u buttons;
+	Bit8u buttons;
 	Bit16u times_pressed[MOUSE_BUTTONS];
 	Bit16u times_released[MOUSE_BUTTONS];
 	Bit16u last_released_x[MOUSE_BUTTONS];
 	Bit16u last_released_y[MOUSE_BUTTONS];
 	Bit16u last_pressed_x[MOUSE_BUTTONS];
 	Bit16u last_pressed_y[MOUSE_BUTTONS];
-	Bit16u shown;
+	Bit16u hidden;
 	float add_x,add_y;
-	Bit16u min_x,max_x,min_y,max_y;
+	Bit16s min_x,max_x,min_y,max_y;
 	float mickey_x,mickey_y;
 	float x,y;
 	button_event event_queue[QUEUE_SIZE];
-	Bit32u events;
+	Bit8u events;//Increase if QUEUE_SIZE >255 (currently 32)
 	Bit16u sub_seg,sub_ofs;
 	Bit16u sub_mask;
 
@@ -109,6 +109,9 @@ static struct {
 	float	mickeysPerPixel_y;
 	float	pixelPerMickey_x;
 	float	pixelPerMickey_y;
+	Bit16u	senv_x_val;
+	Bit16u	senv_y_val;
+	Bit16u	dspeed_val;
 	float	senv_x;
 	float	senv_y;
 	Bit16u  updateRegion_x[2];
@@ -116,10 +119,13 @@ static struct {
 	Bit16u  doubleSpeedThreshold;
 	Bit16u  language;
 	Bit16u  cursorType;
-	Bit16u	oldshown;
+	Bit16u	oldhidden;
 	Bit8u  page;
 	bool enabled;
-
+	bool inhibit_draw;
+	bool timer_in_progress;
+	bool in_UIR;
+	Bit8u mode;
 } mouse;
 
 bool Mouse_SetPS2State(bool use) {
@@ -185,26 +191,44 @@ Bitu PS2_Handler(void) {
 #define X_MICKEY 8
 #define Y_MICKEY 8
 
-#define MOUSE_MOVED 1
+#define MOUSE_HAS_MOVED 1
 #define MOUSE_LEFT_PRESSED 2
 #define MOUSE_LEFT_RELEASED 4
 #define MOUSE_RIGHT_PRESSED 8
 #define MOUSE_RIGHT_RELEASED 16
 #define MOUSE_MIDDLE_PRESSED 32
 #define MOUSE_MIDDLE_RELEASED 64
+#define MOUSE_DELAY 5.0
 
-INLINE void Mouse_AddEvent(Bit16u type) {
+void MOUSE_Limit_Events(Bitu /*val*/) {
+	mouse.timer_in_progress = false;
+	if (mouse.events) {
+		mouse.timer_in_progress = true;
+		PIC_AddEvent(MOUSE_Limit_Events,MOUSE_DELAY);
+		PIC_ActivateIRQ(MOUSE_IRQ);
+	}
+}
+
+INLINE void Mouse_AddEvent(Bit8u type) {
 	if (mouse.events<QUEUE_SIZE) {
-/* Always put the newest element in the front as that the events are 
- * handled backwards (prevents doubleclicks while moving)
- */
-		for(Bitu i = mouse.events ; i ; i--)
-			mouse.event_queue[i] = mouse.event_queue[i-1];
+		if (mouse.events>0) {
+			/* Skip duplicate events */
+			if (type==MOUSE_HAS_MOVED) return;
+			/* Always put the newest element in the front as that the events are 
+			 * handled backwards (prevents doubleclicks while moving)
+			 */
+			for(Bitu i = mouse.events ; i ; i--)
+				mouse.event_queue[i] = mouse.event_queue[i-1];
+		}
 		mouse.event_queue[0].type=type;
 		mouse.event_queue[0].buttons=mouse.buttons;
 		mouse.events++;
 	}
-	PIC_ActivateIRQ(MOUSE_IRQ);
+	if (!mouse.timer_in_progress) {
+		mouse.timer_in_progress = true;
+		PIC_AddEvent(MOUSE_Limit_Events,MOUSE_DELAY);
+		PIC_ActivateIRQ(MOUSE_IRQ);
+	}
 }
 
 // ***************************************************************************
@@ -214,18 +238,16 @@ INLINE void Mouse_AddEvent(Bit16u type) {
 extern void WriteChar(Bit16u col,Bit16u row,Bit8u page,Bit8u chr,Bit8u attr,bool useattr);
 extern void ReadCharAttr(Bit16u col,Bit16u row,Bit8u page,Bit16u * result);
 
-void RestoreCursorBackgroundText()
-{
-	if (mouse.shown) return;
+void RestoreCursorBackgroundText() {
+	if (mouse.hidden || mouse.inhibit_draw) return;
 
 	if (mouse.background) {
-		WriteChar(mouse.backposx,mouse.backposy,0,mouse.backData[0],mouse.backData[1],true);
+		WriteChar(mouse.backposx,mouse.backposy,real_readb(BIOSMEM_SEG,BIOSMEM_CURRENT_PAGE),mouse.backData[0],mouse.backData[1],true);
 		mouse.background = false;
 	}
-};
+}
 
-void DrawCursorText()
-{	
+void DrawCursorText() {	
 	// Restore Background
 	RestoreCursorBackgroundText();
 
@@ -234,15 +256,18 @@ void DrawCursorText()
 	mouse.backposx		= POS_X>>3;
 	mouse.backposy		= POS_Y>>3;
 
+	//use current page (CV program)
+	Bit8u page = real_readb(BIOSMEM_SEG,BIOSMEM_CURRENT_PAGE);
 	Bit16u result;
-	ReadCharAttr(mouse.backposx,mouse.backposy,0,&result);
+
+	ReadCharAttr(mouse.backposx,mouse.backposy,page,&result);
 	mouse.backData[0]	= (Bit8u)(result & 0xFF);
 	mouse.backData[1]	= (Bit8u)(result>>8);
 	mouse.background	= true;
 	// Write Cursor
 	result = (result & mouse.textAndMask) ^ mouse.textXorMask;
-	WriteChar(mouse.backposx,mouse.backposy,0,(Bit8u)(result&0xFF),(Bit8u)(result>>8),true);
-};
+	WriteChar(mouse.backposx,mouse.backposy,page,(Bit8u)(result&0xFF),(Bit8u)(result>>8),true);
+}
 
 // ***************************************************************************
 // Mouse cursor - graphic mode
@@ -250,35 +275,41 @@ void DrawCursorText()
 
 static Bit8u gfxReg3CE[9];
 static Bit8u index3C4,gfxReg3C5;
-void SaveVgaRegisters()
-{
-	for (int i=0; i<9; i++) {
-		IO_Write	(0x3CE,i);
-		gfxReg3CE[i] = IO_Read(0x3CF);
-	}
-	/* Setup some default values in GFX regs that should work */
-	IO_Write(0x3CE,3); IO_Write(0x3Cf,0);				//disable rotate and operation
-	IO_Write(0x3CE,5); IO_Write(0x3Cf,gfxReg3CE[5]&0xf0);	//Force read/write mode 0
+void SaveVgaRegisters() {
+	if (IS_VGA_ARCH) {
+		for (Bit8u i=0; i<9; i++) {
+			IO_Write	(0x3CE,i);
+			gfxReg3CE[i] = IO_Read(0x3CF);
+		}
+		/* Setup some default values in GFX regs that should work */
+		IO_Write(0x3CE,3); IO_Write(0x3Cf,0);				//disable rotate and operation
+		IO_Write(0x3CE,5); IO_Write(0x3Cf,gfxReg3CE[5]&0xf0);	//Force read/write mode 0
 
-	//Set Map to all planes. Celtic Tales
-	index3C4 = IO_Read(0x3c4);  IO_Write(0x3C4,2);
-	gfxReg3C5 = IO_Read(0x3C5); IO_Write(0x3C5,0xF); 
+		//Set Map to all planes. Celtic Tales
+		index3C4 = IO_Read(0x3c4);  IO_Write(0x3C4,2);
+		gfxReg3C5 = IO_Read(0x3C5); IO_Write(0x3C5,0xF);
+	} else if (machine==MCH_EGA) {
+		//Set Map to all planes.
+		IO_Write(0x3C4,2);
+		IO_Write(0x3C5,0xF);
+	}
 }
 
-void RestoreVgaRegisters()
-{
-	for (int i=0; i<9; i++) {
-		IO_Write(0x3CE,i);
-		IO_Write(0x3CF,gfxReg3CE[i]);
-	}
+void RestoreVgaRegisters() {
+	if (IS_VGA_ARCH) {
+		for (Bit8u i=0; i<9; i++) {
+			IO_Write(0x3CE,i);
+			IO_Write(0x3CF,gfxReg3CE[i]);
+		}
 
-	IO_Write(0x3C4,2);
-	IO_Write(0x3C5,gfxReg3C5);
-	IO_Write(0x3C4,index3C4);
+		IO_Write(0x3C4,2);
+		IO_Write(0x3C5,gfxReg3C5);
+		IO_Write(0x3C4,index3C4);
+	}
 }
 
-void ClipCursorArea(Bit16s& x1, Bit16s& x2, Bit16s& y1, Bit16s& y2, Bit16u& addx1, Bit16u& addx2, Bit16u& addy)
-{
+void ClipCursorArea(Bit16s& x1, Bit16s& x2, Bit16s& y1, Bit16s& y2,
+					Bit16u& addx1, Bit16u& addx2, Bit16u& addy) {
 	addx1 = addx2 = addy = 0;
 	// Clip up
 	if (y1<0) {
@@ -299,11 +330,10 @@ void ClipCursorArea(Bit16s& x1, Bit16s& x2, Bit16s& y1, Bit16s& y2, Bit16u& addx
 		addx2 = x2 - mouse.clipx;
 		x2 = mouse.clipx;
 	};
-};
+}
 
-void RestoreCursorBackground()
-{
-	if (mouse.shown) return;
+void RestoreCursorBackground() {
+	if (mouse.hidden || mouse.inhibit_draw) return;
 
 	SaveVgaRegisters();
 	if (mouse.background) {
@@ -329,11 +359,18 @@ void RestoreCursorBackground()
 		mouse.background = false;
 	};
 	RestoreVgaRegisters();
-};
+}
 
 void DrawCursor() {
-	if (mouse.shown) return;
-// Check video page
+	if (mouse.hidden || mouse.inhibit_draw) return;
+	// In Textmode ?
+	if (CurMode->type==M_TEXT) {
+		DrawCursorText();
+		return;
+	}
+
+	// Check video page. Seems to be ignored for text mode. 
+	// hence the text mode handled above this
 	if (real_readb(BIOSMEM_SEG,BIOSMEM_CURRENT_PAGE)!=mouse.page) return;
 // Check if cursor in update region
 /*	if ((POS_X >= mouse.updateRegion_x[0]) && (POS_X <= mouse.updateRegion_x[1]) &&
@@ -349,16 +386,14 @@ void DrawCursor() {
 		 
 	// Get Clipping ranges
 
-	// In Textmode ?
-	if (CurMode->type==M_TEXT) {
-		DrawCursorText();
-		return;
-	}
 
-	mouse.clipx = CurMode->swidth-1;	/* Get from bios ? */
-	mouse.clipy = CurMode->sheight-1;
-	Bit16s xratio = 640 / CurMode->swidth;/* might be vidmode == 0x13?2:1 */
-	if(xratio==0) xratio = 1;
+	mouse.clipx = (Bit16s)((Bits)CurMode->swidth-1);	/* Get from bios ? */
+	mouse.clipy = (Bit16s)((Bits)CurMode->sheight-1);
+
+	/* might be vidmode == 0x13?2:1 */
+	Bit16s xratio = 640;
+	if (CurMode->swidth>0) xratio/=CurMode->swidth;
+	if (xratio==0) xratio = 1;
 	
 	RestoreCursorBackground();
 
@@ -414,8 +449,8 @@ void Mouse_CursorMoved(float xrel,float yrel,float x,float y,bool emulate) {
 	float dx = xrel * mouse.pixelPerMickey_x;
 	float dy = yrel * mouse.pixelPerMickey_y;
 
-	if((fabs(x) > 1.0) || (mouse.senv_x < 1.0)) dx *= mouse.senv_x;
-	if((fabs(y) > 1.0) || (mouse.senv_y < 1.0)) dy *= mouse.senv_y;
+	if((fabs(xrel) > 1.0) || (mouse.senv_x < 1.0)) dx *= mouse.senv_x;
+	if((fabs(yrel) > 1.0) || (mouse.senv_y < 1.0)) dy *= mouse.senv_y;
 	if (useps2callback) dy *= 2;	
 
 	mouse.mickey_x += dx;
@@ -427,9 +462,14 @@ void Mouse_CursorMoved(float xrel,float yrel,float x,float y,bool emulate) {
 		if (CurMode->type == M_TEXT) {
 			mouse.x = x*CurMode->swidth;
 			mouse.y = y*CurMode->sheight * 8 / CurMode->cheight;
-		} else if (mouse.max_x < 2048 || mouse.max_y < 2048 || mouse.max_x != mouse.max_y) {
-			mouse.x = x*mouse.max_x;
-			mouse.y = y*mouse.max_y;
+		} else if ((mouse.max_x < 2048) || (mouse.max_y < 2048) || (mouse.max_x != mouse.max_y)) {
+			if ((mouse.max_x > 0) && (mouse.max_y > 0)) {
+				mouse.x = x*mouse.max_x;
+				mouse.y = y*mouse.max_y;
+			} else {
+				mouse.x += xrel;
+				mouse.y += yrel;
+			}
 		} else { // Games faking relative movement through absolute coordinates. Quite surprising that this actually works..
 			mouse.x += xrel;
 			mouse.y += yrel;
@@ -444,7 +484,7 @@ void Mouse_CursorMoved(float xrel,float yrel,float x,float y,bool emulate) {
 		if (mouse.y > mouse.max_y) mouse.y = mouse.max_y;
 		if (mouse.y < mouse.min_y) mouse.y = mouse.min_y;
 	}
-	Mouse_AddEvent(MOUSE_MOVED);
+	Mouse_AddEvent(MOUSE_HAS_MOVED);
 	DrawCursor();
 }
 
@@ -494,46 +534,52 @@ void Mouse_ButtonReleased(Bit8u button) {
 	mouse.last_released_y[button]=POS_Y;
 }
 
-static void SetMickeyPixelRate(Bit16s px, Bit16s py){
+static void Mouse_SetMickeyPixelRate(Bit16s px, Bit16s py){
 	if ((px!=0) && (py!=0)) {
 		mouse.mickeysPerPixel_x	 = (float)px/X_MICKEY;
 		mouse.mickeysPerPixel_y  = (float)py/Y_MICKEY;
 		mouse.pixelPerMickey_x	 = X_MICKEY/(float)px;
 		mouse.pixelPerMickey_y 	 = Y_MICKEY/(float)py;	
 	}
-};
-static void SetSensitivity(Bit16s px, Bit16s py){
+}
+
+static void Mouse_SetSensitivity(Bit16u px, Bit16u py, Bit16u dspeed){
 	if(px>100) px=100;
 	if(py>100) py=100;
+	if(dspeed>100) dspeed=100;
+	// save values
+	mouse.senv_x_val=px;
+	mouse.senv_y_val=py;
+	mouse.dspeed_val=dspeed;
 	if ((px!=0) && (py!=0)) {
 		px--;  //Inspired by cutemouse 
 		py--;  //Although their cursor update routine is far more complex then ours
 		mouse.senv_x=(static_cast<float>(px)*px)/3600.0f +1.0f/3.0f;
 		mouse.senv_y=(static_cast<float>(py)*py)/3600.0f +1.0f/3.0f;
      }
-};
+}
 
 
-static void mouse_reset_hardware(void){
+static void Mouse_ResetHardware(void){
 	PIC_SetIRQMask(MOUSE_IRQ,false);
-};
+}
 
-void Mouse_NewVideoMode(void)
-{ //Does way to much. Many of this stuff should be moved to mouse_reset one day
-	if(MOUSE_IRQ > 7) {
-		real_writed(0,((0x70+MOUSE_IRQ-8)<<2),CALLBACK_RealPointer(call_int74));
-	} else {
-		real_writed(0,((0x8+MOUSE_IRQ)<<2),CALLBACK_RealPointer(call_int74));
-	}
-	mouse.shown = 1;//Disabled as ida doesn't have mousecursor anymore
-	//enabled again as it seems to be a bug in ida4
+//Does way to much. Many things should be moved to mouse reset one day
+void Mouse_NewVideoMode(void) {
+	mouse.inhibit_draw=false;
 	/* Get the correct resolution from the current video mode */
-	Bitu mode=mem_readb(BIOS_VIDEO_MODE);
+	Bit8u mode=mem_readb(BIOS_VIDEO_MODE);
+	if(mode == mouse.mode) {LOG(LOG_MOUSE,LOG_NORMAL)("New video is the same as the old"); /*return;*/}
 	switch (mode) {
 	case 0x00:
 	case 0x01:
 	case 0x02:
-	case 0x03:
+	case 0x03: {
+		Bitu rows=real_readb(BIOSMEM_SEG,BIOSMEM_NB_ROWS);
+		if ((rows==0) || (rows>250)) rows=25-1;
+		mouse.max_y=8*(rows+1)-1;
+		break;
+	}
 	case 0x04:
 	case 0x05:
 	case 0x06:
@@ -555,21 +601,19 @@ void Mouse_NewVideoMode(void)
 		mouse.max_y=479;
 		break;
 	default:
-		mouse.max_y=199;
 		LOG(LOG_MOUSE,LOG_ERROR)("Unhandled videomode %X on reset",mode);
-		// Hide mouse cursor on non supported modi. Pirates Gold
-		mouse.shown = 1;
-		break;
-	} 
+		mouse.inhibit_draw=true;
+		return;
+	}
+	mouse.mode = mode;
+	mouse.hidden = 1;
 	mouse.max_x = 639;
 	mouse.min_x = 0;
 	mouse.min_y = 0;
-	// Dont set max coordinates here. it is done by SetResolution!
-	mouse.x = static_cast<float>((mouse.max_x + 1)/ 2);
-	mouse.y = static_cast<float>((mouse.max_y + 1)/ 2);
+
 	mouse.events = 0;
-	mouse.mickey_x = 0;
-	mouse.mickey_y = 0;
+	mouse.timer_in_progress = false;
+	PIC_RemoveEvents(MOUSE_Limit_Events);
 
 	mouse.hotx		 = 0;
 	mouse.hoty		 = 0;
@@ -587,43 +631,47 @@ void Mouse_NewVideoMode(void)
 	mouse.updateRegion_y[1] = 1;
 	mouse.cursorType = 0;
 	mouse.enabled=true;
-	mouse.oldshown=1;
+	mouse.oldhidden=1;
 
-	SetMickeyPixelRate(8,16);
 	oldmouseX = static_cast<Bit16s>(mouse.x);
 	oldmouseY = static_cast<Bit16s>(mouse.y);
+
+
 }
 
-static void mouse_reset(void) {
-//Much to empty Mouse_NewVideoMode contains stuff that should be in here
-
+//Much too empty, Mouse_NewVideoMode contains stuff that should be in here
+static void Mouse_Reset(void) {
 	/* Remove drawn mouse Legends of Valor */
 	if (CurMode->type!=M_TEXT) RestoreCursorBackground();
 	else RestoreCursorBackgroundText();
-	mouse.shown = 1;
+	mouse.hidden = 1;
 
 	Mouse_NewVideoMode();
+	Mouse_SetMickeyPixelRate(8,16);
 
-   	mouse.sub_mask=0;
+	mouse.mickey_x = 0;
+	mouse.mickey_y = 0;
 
-	mouse.senv_x=1.0;
-	mouse.senv_y=1.0;
+	// Dont set max coordinates here. it is done by SetResolution!
+	mouse.x = static_cast<float>((mouse.max_x + 1)/ 2);
+	mouse.y = static_cast<float>((mouse.max_y + 1)/ 2);
+	mouse.sub_mask = 0;
+	mouse.in_UIR = false;
 }
 
 static Bitu INT33_Handler(void) {
-
 //	LOG(LOG_MOUSE,LOG_NORMAL)("MOUSE: %04X %X %X %d %d",reg_ax,reg_bx,reg_cx,POS_X,POS_Y);
 	switch (reg_ax) {
 	case 0x00:	/* Reset Driver and Read Status */
-		mouse_reset_hardware(); /* fallthrough */
+		Mouse_ResetHardware(); /* fallthrough */
 	case 0x21:	/* Software Reset */
 		reg_ax=0xffff;
 		reg_bx=MOUSE_BUTTONS;
-		mouse_reset();
+		Mouse_Reset();
 		Mouse_AutoLock(true);
 		break;
 	case 0x01:	/* Show Mouse */
-		if(mouse.shown) mouse.shown--;
+		if(mouse.hidden) mouse.hidden--;
 		Mouse_AutoLock(true);
 		DrawCursor();
 		break;
@@ -631,7 +679,7 @@ static Bitu INT33_Handler(void) {
 		{
 			if (CurMode->type!=M_TEXT) RestoreCursorBackground();
 			else RestoreCursorBackgroundText();
-			mouse.shown++;
+			mouse.hidden++;
 		}
 		break;
 	case 0x03:	/* Return position and Button Status */
@@ -643,21 +691,20 @@ static Bitu INT33_Handler(void) {
 		/* If position isn't different from current position
 		 * don't change it then. (as position is rounded so numbers get
 		 * lost when the rounded number is set) (arena/simulation Wolf) */
-		if(reg_cx >= mouse.max_x) mouse.x = static_cast<float>(mouse.max_x);
-		else if (mouse.min_x >= reg_cx) mouse.x = static_cast<float>(mouse.min_x); 
-		else if (reg_cx != POS_X) mouse.x = static_cast<float>(reg_cx);
+		if ((Bit16s)reg_cx >= mouse.max_x) mouse.x = static_cast<float>(mouse.max_x);
+		else if (mouse.min_x >= (Bit16s)reg_cx) mouse.x = static_cast<float>(mouse.min_x); 
+		else if ((Bit16s)reg_cx != POS_X) mouse.x = static_cast<float>(reg_cx);
 
-		if(reg_dx >= mouse.max_y) mouse.y = static_cast<float>(mouse.max_y);
-		else if (mouse.min_y >= reg_dx) mouse.y = static_cast<float>(mouse.min_y); 
-		else if (reg_dx != POS_Y) mouse.y = static_cast<float>(reg_dx);
+		if ((Bit16s)reg_dx >= mouse.max_y) mouse.y = static_cast<float>(mouse.max_y);
+		else if (mouse.min_y >= (Bit16s)reg_dx) mouse.y = static_cast<float>(mouse.min_y); 
+		else if ((Bit16s)reg_dx != POS_Y) mouse.y = static_cast<float>(reg_dx);
 		DrawCursor();
 		break;
 	case 0x05:	/* Return Button Press Data */
 		{
 			Bit16u but=reg_bx;
 			reg_ax=mouse.buttons;
-			reg_bx++;
-			if (but>=MOUSE_BUTTONS) break;
+			if (but>=MOUSE_BUTTONS) but = MOUSE_BUTTONS - 1;
 			reg_cx=mouse.last_pressed_x[but];
 			reg_dx=mouse.last_pressed_y[but];
 			reg_bx=mouse.times_pressed[but];
@@ -668,8 +715,7 @@ static Bitu INT33_Handler(void) {
 		{
 			Bit16u but=reg_bx;
 			reg_ax=mouse.buttons;
-			reg_bx++;
-			if (but>=MOUSE_BUTTONS) break;
+			if (but>=MOUSE_BUTTONS) but = MOUSE_BUTTONS - 1;
 			reg_cx=mouse.last_released_x[but];
 			reg_dx=mouse.last_released_y[but];
 			reg_bx=mouse.times_released[but];
@@ -680,7 +726,7 @@ static Bitu INT33_Handler(void) {
 		{	//lemmings set 1-640 and wants that. iron seeds set 0-640 but doesn't like 640
 			//Iron seed works if newvideo mode with mode 13 sets 0-639
 			//Larry 6 actually wants newvideo mode with mode 13 to set it to 0-319
-			Bits max,min;
+			Bit16s max,min;
 			if ((Bit16s)reg_cx<(Bit16s)reg_dx) { min=(Bit16s)reg_cx;max=(Bit16s)reg_dx;}
 			else { min=(Bit16s)reg_dx;max=(Bit16s)reg_cx;}
 			mouse.min_x=min;
@@ -697,7 +743,7 @@ static Bitu INT33_Handler(void) {
 		{	// not sure what to take instead of the CurMode (see case 0x07 as well)
 			// especially the cases where sheight= 400 and we set it with the mouse_reset to 200
 			//disabled it at the moment. Seems to break syndicate who want 400 in mode 13
-			Bits max,min;
+			Bit16s max,min;
 			if ((Bit16s)reg_cx<(Bit16s)reg_dx) { min=(Bit16s)reg_cx;max=(Bit16s)reg_dx;}
 			else { min=(Bit16s)reg_dx;max=(Bit16s)reg_cx;}
 			mouse.min_y=min;
@@ -728,6 +774,12 @@ static Bitu INT33_Handler(void) {
 		mouse.textAndMask = reg_cx;
 		mouse.textXorMask = reg_dx;
 		break;
+	case 0x0b:	/* Read Motion Data */
+		reg_cx=(Bit16s)(mouse.mickey_x*mouse.mickeysPerPixel_x);
+		reg_dx=(Bit16s)(mouse.mickey_y*mouse.mickeysPerPixel_y);
+		mouse.mickey_x=0;
+		mouse.mickey_y=0;
+		break;
 	case 0x0c:	/* Define interrupt subroutine parameters */
 		mouse.sub_mask=reg_cx;
 		mouse.sub_seg=SegValue(es);
@@ -735,13 +787,7 @@ static Bitu INT33_Handler(void) {
 		Mouse_AutoLock(true); //Some games don't seem to reset the mouse before using
 		break;
 	case 0x0f:	/* Define mickey/pixel rate */
-		SetMickeyPixelRate(reg_cx,reg_dx);
-		break;
-	case 0x0B:	/* Read Motion Data */
-		reg_cx=(Bit16s)(mouse.mickey_x*mouse.mickeysPerPixel_x);
-		reg_dx=(Bit16s)(mouse.mickey_y*mouse.mickeysPerPixel_y);
-		mouse.mickey_x=0;
-		mouse.mickey_y=0;
+		Mouse_SetMickeyPixelRate(reg_cx,reg_dx);
 		break;
 	case 0x10:      /* Define screen region for updating */
 		mouse.updateRegion_x[0]=reg_cx;
@@ -789,21 +835,17 @@ static Bitu INT33_Handler(void) {
 		}
 		break;
 	case 0x1a:	/* Set mouse sensitivity */
-		SetSensitivity(reg_bx,reg_cx);
-
-		LOG(LOG_MOUSE,LOG_WARN)("Set sensitivity used with %d %d",reg_bx,reg_cx);
-
 		// ToDo : double mouse speed value
+		Mouse_SetSensitivity(reg_bx,reg_cx,reg_dx);
+
+		LOG(LOG_MOUSE,LOG_WARN)("Set sensitivity used with %d %d (%d)",reg_bx,reg_cx,reg_dx);
 		break;
 	case 0x1b:	/* Get mouse sensitivity */
-		reg_bx = Bit16s((60.0* sqrt(mouse.senv_x- (1.0/3.0)) ) +1.0);
-
-		reg_cx = Bit16s((60.0* sqrt(mouse.senv_y- (1.0/3.0)) ) +1.0);
+		reg_bx = mouse.senv_x_val;
+		reg_cx = mouse.senv_y_val;
+		reg_dx = mouse.dspeed_val;
 
 		LOG(LOG_MOUSE,LOG_WARN)("Get sensitivity %d %d",reg_bx,reg_cx);
-
-		// ToDo : double mouse speed value
-		reg_dx = 64;
 		break;
 	case 0x1c:	/* Set interrupt rate */
 		/* Can't really set a rate this is host determined */
@@ -819,12 +861,12 @@ static Bitu INT33_Handler(void) {
 		reg_bx=0;
 		SegSet16(es,0);	   
 		mouse.enabled=false; /* Just for reporting not doing a thing with it */
-		mouse.oldshown=mouse.shown;
-		mouse.shown=1;
+		mouse.oldhidden=mouse.hidden;
+		mouse.hidden=1;
 		break;
 	case 0x20:	/* Enable Mousedriver */
 		mouse.enabled=true;
-		mouse.shown=mouse.oldshown;
+		mouse.hidden=mouse.oldhidden;
 		break;
 	case 0x22:      /* Set language for messages */
  			/*
@@ -849,15 +891,16 @@ static Bitu INT33_Handler(void) {
 	case 0x24:	/* Get Software version and mouse type */
 		reg_bx=0x805;	//Version 8.05 woohoo 
 		reg_ch=0x04;	/* PS/2 type */
-		reg_cl=0;//MOUSE_IRQ;		/* Hmm ps2 irq 0!!!! */
+		reg_cl=0;		/* PS/2 (unused) */
 		break;
 	case 0x26: /* Get Maximum virtual coordinates */
 		reg_bx=(mouse.enabled ? 0x0000 : 0xffff);
-		reg_cx=mouse.max_x;
-		reg_dx=mouse.max_y;
+		reg_cx=(Bit16u)mouse.max_x;
+		reg_dx=(Bit16u)mouse.max_y;
 		break;
 	default:
 		LOG(LOG_MOUSE,LOG_ERROR)("Mouse Function %04X not implemented!",reg_ax);
+		break;
 	}
 	return CBRET_NONE;
 }
@@ -936,6 +979,8 @@ static Bitu INT74_Handler(void) {
 			CPU_Push16(RealOff(CALLBACK_RealPointer(int74_ret_callback)));
 			SegSet16(cs, mouse.sub_seg);
 			reg_ip = mouse.sub_ofs;
+			if(mouse.in_UIR) LOG(LOG_MOUSE,LOG_ERROR)("Already in UIR!");
+			mouse.in_UIR = true;
 		} else if (useps2callback) {
 			CPU_Push16(RealSeg(CALLBACK_RealPointer(int74_ret_callback)));
 			CPU_Push16(RealOff(CALLBACK_RealPointer(int74_ret_callback)));
@@ -952,13 +997,17 @@ static Bitu INT74_Handler(void) {
 }
 
 Bitu MOUSE_UserInt_CB_Handler(void) {
+	mouse.in_UIR = false;
 	if (mouse.events) {
-		PIC_ActivateIRQ(MOUSE_IRQ);
+		if (!mouse.timer_in_progress) {
+			mouse.timer_in_progress = true;
+			PIC_AddEvent(MOUSE_Limit_Events,MOUSE_DELAY);
+		}
 	}
 	return CBRET_NONE;
 }
 
-void MOUSE_Init(Section* sec) {
+void MOUSE_Init(Section* /*sec*/) {
 	// Callback for mouse interrupt 0x33
 	call_int33=CALLBACK_Allocate();
 //	RealPt i33loc=RealMake(CB_SEG+1,(call_int33*CB_SIZE)-0x10);
@@ -1014,12 +1063,15 @@ void MOUSE_Init(Section* sec) {
 	ps2_callback=CALLBACK_RealPointer(call_ps2);
 
 	memset(&mouse,0,sizeof(mouse));
-	mouse.shown = 1; //Hide mouse on startup
+	mouse.hidden = 1; //Hide mouse on startup
+	mouse.timer_in_progress = false;
+	mouse.mode = 0xFF; //Non existing mode
 
    	mouse.sub_mask=0;
 	mouse.sub_seg=0x6362;	// magic value
 	mouse.sub_ofs=0;
 
-	mouse_reset_hardware();
-	mouse_reset();
+	Mouse_ResetHardware();
+	Mouse_Reset();
+	Mouse_SetSensitivity(50,50,50);
 }
